@@ -91,7 +91,7 @@ show_data_sources (DsnConfigPrivate *priv)
 	gnome_db_config_free_data_source_list (dsn_list);
 
 	/* display the new data model */
-	gnome_db_grid_set_gda_model (GNOME_DB_GRID (priv->dsn_list), NULL, model);
+	gnome_db_data_widget_set_gda_model (GNOME_DB_DATA_WIDGET (priv->dsn_list), NULL, model);
 	g_object_unref (G_OBJECT (model));
 }
 
@@ -184,17 +184,17 @@ dsn_config_new (void)
 
 	/* create the data source list */
 	priv->dsn_list = gnome_db_grid_new ();
-	gnome_db_data_widget_global_show_actions (GNOME_DB_DATA_WIDGET (priv->dsn_list), FALSE);
-	gnome_db_data_widget_column_set_editable (GNOME_DB_DATA_WIDGET (priv->dsn_list), 0, FALSE);
+ 	gnome_db_data_widget_column_set_editable (GNOME_DB_DATA_WIDGET (priv->dsn_list), 0, FALSE);
+	g_object_set (G_OBJECT (priv->dsn_list), "info_cell_visible", FALSE, "actions_visible", FALSE, NULL);
 
 	title = g_strdup_printf ("<b>%s</b>\n%s", _("Data Sources"),
 				 _("Configured data sources in the system"));
-	gnome_db_grid_set_title (GNOME_DB_GRID (priv->dsn_list), title);
+	gnome_db_data_widget_set_title (GNOME_DB_DATA_WIDGET (priv->dsn_list), title);
 	g_free (title);
-
-	gnome_db_grid_set_show_title (GNOME_DB_GRID (priv->dsn_list), TRUE);
-	gnome_db_grid_set_show_title_icon (GNOME_DB_GRID (priv->dsn_list), TRUE);
-	gnome_db_grid_set_title_icon_from_stock (GNOME_DB_GRID (priv->dsn_list), GNOME_DB_STOCK_DATABASE, GTK_ICON_SIZE_BUTTON);
+	
+	gnome_db_data_widget_set_show_title (GNOME_DB_DATA_WIDGET (priv->dsn_list), TRUE);
+	gnome_db_data_widget_set_title_icon_from_stock (GNOME_DB_DATA_WIDGET (priv->dsn_list), 
+							GNOME_DB_STOCK_DATABASE, GTK_ICON_SIZE_BUTTON);
 	gtk_widget_show (priv->dsn_list);
 	g_signal_connect (G_OBJECT (priv->dsn_list), "double_clicked",
 			  G_CALLBACK (list_double_clicked_cb), dsn);
@@ -249,7 +249,7 @@ dsn_config_edit_properties (GtkWidget *dsn)
 	if (!selection)
 		return;
 
-	model = gnome_db_grid_get_gda_model (GNOME_DB_GRID (priv->dsn_list));
+	model = gnome_db_data_widget_get_gda_model (GNOME_DB_DATA_WIDGET (priv->dsn_list));
 	if (!GDA_IS_DATA_MODEL (model))
 		return;
 
@@ -273,7 +273,7 @@ foreach_remove_cb (GnomeDbGrid *grid, gint row, gpointer user_data)
 	gchar *dsn;
 
 	priv = (DsnConfigPrivate *) user_data;
-	model = gnome_db_grid_get_gda_model (GNOME_DB_GRID (priv->dsn_list));
+	model = gnome_db_data_widget_get_gda_model (GNOME_DB_DATA_WIDGET (priv->dsn_list));
 	if (!GDA_IS_DATA_MODEL (model))
 		return;
 
