@@ -23,7 +23,7 @@
 #include <gtk/gtkdialog.h>
 #include <gtk/gtkstock.h>
 #include <libgnomedb/libgnomedb.h>
-#include <libgnomedb/db-shell.h>
+#include "db-shell.h"
 #include <libgnomeui/gnome-client.h>
 #include <libgnomedb/gnome-db-i18n.h>
 #include <libgnomeui/gnome-window-icon.h>
