@@ -73,7 +73,7 @@ show_providers (ProviderConfigPrivate *priv)
 	gda_config_free_provider_list (prv_list);
 
 	/* display the new data model */
-	gnome_db_grid_set_gda_model (GNOME_DB_GRID (priv->provider_list), NULL, model);
+	gnome_db_data_widget_set_gda_model (GNOME_DB_DATA_WIDGET (priv->provider_list), NULL, model);
 	g_object_unref (G_OBJECT (model));
 }
 
@@ -103,17 +103,16 @@ provider_config_new (void)
 
 	/* create the provider list */
 	priv->provider_list = gnome_db_grid_new ();
-	gnome_db_data_widget_global_show_actions (GNOME_DB_DATA_WIDGET (priv->provider_list), FALSE);
 	gnome_db_data_widget_column_set_editable (GNOME_DB_DATA_WIDGET (priv->provider_list), 0, FALSE);
+	g_object_set (G_OBJECT (priv->provider_list), "info_cell_visible", FALSE, "actions_visible", FALSE, NULL);
 
 	title = g_strdup_printf ("<b>%s</b>\n%s", _("Providers"),
 				 _("Installed providers"));
-	gnome_db_grid_set_title (GNOME_DB_GRID (priv->provider_list), title);
+	gnome_db_data_widget_set_title (GNOME_DB_DATA_WIDGET (priv->provider_list), title);
 	g_free (title);
 
-	gnome_db_grid_set_show_title (GNOME_DB_GRID (priv->provider_list), TRUE);
-	gnome_db_grid_set_show_title_icon (GNOME_DB_GRID (priv->provider_list), TRUE);
-	gnome_db_grid_set_title_icon_from_stock (GNOME_DB_GRID (priv->provider_list), GNOME_DB_STOCK_DATABASE, GTK_ICON_SIZE_BUTTON);
+	gnome_db_data_widget_set_show_title (GNOME_DB_DATA_WIDGET (priv->provider_list), TRUE);
+	gnome_db_data_widget_set_title_icon_from_stock (GNOME_DB_DATA_WIDGET (priv->provider_list), GNOME_DB_STOCK_DATABASE, GTK_ICON_SIZE_BUTTON);
 	gtk_widget_show (priv->provider_list);
 	gtk_table_attach (GTK_TABLE (table), priv->provider_list, 0, 1, 0, 1,
 			  GTK_FILL | GTK_SHRINK | GTK_EXPAND,
