@@ -38,7 +38,7 @@ int
 main (int argc, char **argv)
 {
 	GtkWidget *mainwin, *vbox, *menu, *nb, *label, *page;
-	GnomeDbDict *dictionnary;
+	GnomeDbDict *dictionary;
 	BrowserConfig *config;
 	GdkPixbuf *icon;
 	
@@ -50,9 +50,9 @@ main (int argc, char **argv)
 
 	/* Browser config */
 	config = g_new0 (BrowserConfig, 1);
-	dictionnary = GNOME_DB_DICT (gnome_db_dict_new ());
-	config->dict = dictionnary;
-	g_object_set (G_OBJECT (gnome_db_dict_get_server (dictionnary)), "with_functions", TRUE, NULL);
+	dictionary = GNOME_DB_DICT (gnome_db_dict_new ());
+	config->dict = dictionary;
+	g_object_set (G_OBJECT (gnome_db_dict_get_server (dictionary)), "with_functions", TRUE, NULL);
 
 	/* Application init */
 	if (argc > 1) {
@@ -116,7 +116,7 @@ main (int argc, char **argv)
 	gtk_widget_show_all (mainwin);
 	
 	gtk_main ();
-	g_object_unref (G_OBJECT (dictionnary));
+	g_object_unref (G_OBJECT (dictionary));
 
 	return 0;
 }
@@ -137,8 +137,8 @@ load_file (BrowserConfig *config)
 			gchar *str;
 
 			str = g_strdup_printf ("<b>%s</b>\n\n%s",
-					       _("Do you want to build the database dictionnary?"),
-					       _("The database dictionnary must be built before the database can be browsed, "
+					       _("Do you want to build the database dictionary?"),
+					       _("The database dictionary must be built before the database can be browsed, "
 						 "the process only takes a few minutes.\n\n"
 						 "It is saved and a re-building will only be necessary "
 						 "when the database structures changes."));
@@ -151,7 +151,7 @@ load_file (BrowserConfig *config)
 			if (gtk_dialog_run (GTK_DIALOG (msg)) == GTK_RESPONSE_YES) 
 				open_connection (config, TRUE);
 			else {
-				g_print ("Can't build the dictionnary...\n");
+				g_print ("Can't build the dictionary...\n");
 				exit (1);
 			}
 			gtk_widget_destroy (msg);
@@ -159,7 +159,7 @@ load_file (BrowserConfig *config)
 			error = NULL;
 		}
 		else {
-			g_print ("Error loading dictionnary file:\n%s\n",
+			g_print ("Error loading dictionary file:\n%s\n",
 				 error ? error->message : _("unreported error"));
 			exit (1);
 		}
@@ -325,7 +325,7 @@ update_dbms_data_cb (GtkWidget *wid, BrowserConfig *config)
 			msg = gtk_message_dialog_new (GTK_WINDOW (config->mainwin), 
 						      GTK_DIALOG_DESTROY_WITH_PARENT,
 						      GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
-						      _("Error saving dictionnary file:\n%s\n"),
+						      _("Error saving dictionary file:\n%s\n"),
 						      error->message);
 			gtk_dialog_run (GTK_DIALOG (msg));
 			gtk_widget_destroy (msg);
