@@ -1,6 +1,6 @@
 /* gnome-db-base.c
  *
- * Copyright (C) 2003 - 2004 Vivien Malerba
+ * Copyright (C) 2003 - 2005 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -332,7 +332,7 @@ gnome_db_base_set_dict (GnomeDbBase *base, GnomeDbDict *dict)
 {
 	g_return_if_fail (base && IS_GNOME_DB_BASE (base));
 	g_return_if_fail (base->priv);
-	g_return_if_fail (dict && IS_GNOME_DB_DICT (dict));
+	g_return_if_fail (!dict || IS_GNOME_DB_DICT (dict));
 
 	if (base->priv->dict) {
 		g_object_remove_weak_pointer (G_OBJECT (base->priv->dict),
@@ -340,7 +340,7 @@ gnome_db_base_set_dict (GnomeDbBase *base, GnomeDbDict *dict)
 		base->priv->dict = NULL;
 	}
 
-	base->priv->dict = dict;
+	base->priv->dict = ASSERT_DICT (dict);
 	g_object_add_weak_pointer (G_OBJECT (base->priv->dict),
 				   (gpointer) & (base->priv->dict));
 }
