@@ -173,7 +173,7 @@ gnome_db_basic_form_new_in_layout (GnomeDbDict *dict, GnomeDbDataSet *data_set,
 	GObject *obj;
 	GnomeDbBasicForm *form;
 
-	g_return_val_if_fail (dict && IS_GNOME_DB_DICT (dict), NULL);
+	g_return_val_if_fail (!dict || IS_GNOME_DB_DICT (dict), NULL);
 	if (data_set) {
 		GError *error = NULL;
 		if (! gnome_db_data_set_is_coherent (data_set, &error)) {
@@ -186,7 +186,7 @@ gnome_db_basic_form_new_in_layout (GnomeDbDict *dict, GnomeDbDataSet *data_set,
 	obj = g_object_new (GNOME_DB_BASIC_FORM_TYPE, NULL);
 	form = GNOME_DB_BASIC_FORM (obj);
 
-	form->priv->dict = dict;
+	form->priv->dict = ASSERT_DICT (dict);
 	
 	/* data_set */
 	if (data_set) {
@@ -194,12 +194,12 @@ gnome_db_basic_form_new_in_layout (GnomeDbDict *dict, GnomeDbDataSet *data_set,
 		g_object_ref (data_set);
 	}
 	else
-		form->priv->data_set = GNOME_DB_DATA_SET (gnome_db_data_set_new (dict, NULL));
+		form->priv->data_set = GNOME_DB_DATA_SET (gnome_db_data_set_new (ASSERT_DICT (dict), NULL));
 
 	g_object_weak_ref (G_OBJECT (form->priv->dict),
 			   (GWeakNotify) gnome_db_dict_weak_notify, form);
 
-	gnome_db_basic_form_initialize (form, dict, layout, box_widgets);
+	gnome_db_basic_form_initialize (form, ASSERT_DICT (dict), layout, box_widgets);
 
 	return GTK_WIDGET (obj);
 }
