@@ -523,7 +523,7 @@ gnome_db_combo_undef_selected (GnomeDbCombo *combo)
 		return TRUE;
 	
 	g_object_get (G_OBJECT (combo->priv->proxy), "prepend_null_entry", &has_undef_choice, NULL);
-	if (!has_undef_choice || (active_row == 0))
+	if (has_undef_choice && (active_row == 0))
 		return TRUE;
 
 	return FALSE;
