@@ -1200,20 +1200,37 @@ gnome_db_condition_dump (GnomeDbCondition *condition, guint offset)
 				 str, condition_type_to_str (condition->priv->type), 
 				 condition, gnome_db_base_get_id (GNOME_DB_BASE (condition)), condition->priv->query);
 		if (gnome_db_condition_is_active (GNOME_DB_REFERER (condition)))
-			g_print ("Active\n");
+			g_print ("Active");
 		else
-			g_print (D_COL_ERR "Non active\n" D_COL_NOR);
-		g_print ("\n");
+			g_print (D_COL_ERR "Non active" D_COL_NOR);
+
+		for (i=0; i<3; i++) {
+			if (i == 0)
+				g_print (" (");
+			else
+				g_print (", ");
+			if (condition->priv->ops[i]) {
+				GnomeDbBase *ref;
+
+				ref = gnome_db_ref_base_get_ref_object (condition->priv->ops[i]);
+				if (ref) {
+					gchar *id = gnome_db_xml_storage_get_xml_id (GNOME_DB_XML_STORAGE (ref));
+					g_print ("'%s'=%p", id, ref);
+					g_free (id);
+				}
+				else
+					g_print ("_null");	
+			}
+			else
+				g_print ("NULL");
+		}
+		g_print (")\n");
+
 		children = condition->priv->cond_children;
 		while (children) {
 			gnome_db_condition_dump (GNOME_DB_CONDITION (children->data), offset+5);
 			children = g_slist_next (children);
 		}
-		for (i=0; i<3; i++)
-			if (condition->priv->ops[i]) {
-				g_print ("%s" D_COL_H1 "Field %d:\n" D_COL_NOR, str, i);
-				gnome_db_base_dump (GNOME_DB_BASE (condition->priv->ops[i]), offset+5);
-			}
 	}
         else
                 g_print ("%s" D_COL_ERR "Using finalized object %p" D_COL_NOR, str, condition);
@@ -1615,15 +1632,15 @@ gnome_db_condition_render_as_sql (GnomeDbRenderer *iface, GnomeDbDataSet *contex
 	case GNOME_DB_CONDITION_LEAF_EQUAL:
 		link = "=";
 		if (IS_GNOME_DB_QF_VALUE (ops[GNOME_DB_CONDITION_OP_RIGHT]) &&
-		    gnome_db_qf_value_is_value_null (GNOME_DB_QF_VALUE (ops[GNOME_DB_CONDITION_OP_RIGHT]), context) &&
-		    ! gnome_db_qf_value_is_parameter (GNOME_DB_QF_VALUE (ops[GNOME_DB_CONDITION_OP_RIGHT])))
-			link = " IS ";
+		    context && 
+		    gnome_db_qf_value_is_value_null (GNOME_DB_QF_VALUE (ops[GNOME_DB_CONDITION_OP_RIGHT]), context))
+				link = " IS ";
 		break;
 	case GNOME_DB_CONDITION_LEAF_DIFF:
 		link = "!=";
 		if (IS_GNOME_DB_QF_VALUE (ops[GNOME_DB_CONDITION_OP_RIGHT]) &&
-		    gnome_db_qf_value_is_value_null (GNOME_DB_QF_VALUE (ops[GNOME_DB_CONDITION_OP_RIGHT]), context) &&
-		    ! gnome_db_qf_value_is_parameter (GNOME_DB_QF_VALUE (ops[GNOME_DB_CONDITION_OP_RIGHT])))
+		    context && 
+		    gnome_db_qf_value_is_value_null (GNOME_DB_QF_VALUE (ops[GNOME_DB_CONDITION_OP_RIGHT]), context))
 			link = " IS NOT ";
 		break;
 	case GNOME_DB_CONDITION_LEAF_SUP:
