@@ -21,9 +21,9 @@
  */
 
 #include "gnome-db-i18n.h"
-#include <gtk/gtkentry.h>
-#include <gtk/gtktable.h>
+#include <gtk/gtk.h>
 #include <libgnomedb/gnome-db-util.h>
+#include <libgnomedb/gnome-db-grid.h>
 #include "gnome-db-connection-properties.h"
 
 #define PARENT_TYPE GTK_TYPE_VBOX
@@ -83,9 +83,10 @@ static void
 gnome_db_connection_properties_init (GnomeDbConnectionProperties *props,
 				     GnomeDbConnectionPropertiesClass *klass)
 {
-	GtkWidget *frame;
 	GtkWidget *table;
 	GtkWidget *label;
+	GtkWidget *hbox;
+	gchar *str;
 
 	g_return_if_fail (GNOME_DB_IS_CONNECTION_PROPERTIES (props));
 
@@ -94,11 +95,22 @@ gnome_db_connection_properties_init (GnomeDbConnectionProperties *props,
 	props->priv->cnc = NULL;
 
 	/* create widgets */
-	frame = gnome_db_new_frame_widget (_("Settings"));
-	gtk_box_pack_start (GTK_BOX (props), frame, TRUE, TRUE, 2);
+	str = g_strdup_printf ("<b>%s:</b>", _("Settings"));
+	label = gtk_label_new ("");
+	gtk_label_set_markup (GTK_LABEL (label), str);
+	gtk_misc_set_alignment (GTK_MISC (label), 0., -1);
+	g_free (str);
+	gtk_box_pack_start (GTK_BOX (props), label, FALSE, TRUE, 2);
+	
+	hbox = gtk_hbox_new (FALSE, 0); /* HIG */
+	gtk_box_pack_start (GTK_BOX (props), hbox, FALSE, FALSE, 0);
+	gtk_widget_show (hbox);
+	label = gtk_label_new ("    ");
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+	gtk_widget_show (label);
 
 	table = gnome_db_new_table_widget (6, 3, FALSE);
-	gtk_container_add (GTK_CONTAINER (frame), table);
+	gtk_box_pack_start (GTK_BOX (hbox), table, TRUE, TRUE, 0);	
 
 	label = gnome_db_new_label_widget (_("Data source name"));
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 2, 2);
@@ -125,11 +137,24 @@ gnome_db_connection_properties_init (GnomeDbConnectionProperties *props,
 			  1, 2, 3, 4, GTK_FILL, GTK_FILL, 2, 2);
 
 	/* create feature list */
-	frame = gnome_db_new_frame_widget (_("Features"));
-	gtk_box_pack_start (GTK_BOX (props), frame, TRUE, TRUE, 2);
+	str = g_strdup_printf ("<b>%s:</b>", _("Features"));
+	label = gtk_label_new ("");
+	gtk_label_set_markup (GTK_LABEL (label), str);
+	gtk_misc_set_alignment (GTK_MISC (label), 0., -1);
+	g_free (str);
+	gtk_box_pack_start (GTK_BOX (props), label, FALSE, TRUE, 2);
 
-	props->priv->feature_list = gnome_db_new_grid_widget (NULL);
-	gtk_container_add (GTK_CONTAINER (frame), props->priv->feature_list);
+	hbox = gtk_hbox_new (FALSE, 0); /* HIG */
+	gtk_box_pack_start (GTK_BOX (props), hbox, FALSE, FALSE, 0);
+	gtk_widget_show (hbox);
+	label = gtk_label_new ("    ");
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+	gtk_widget_show (label);
+
+	/* props->priv->feature_list = gnome_db_new_grid_widget (NULL); */
+	/* gnome_db_grid_set_show_title (GNOME_DB_GRID (props->priv->feature_list), FALSE); */
+	props->priv->feature_list = gtk_label_new ("Not yet implemented");
+	gtk_box_pack_start (GTK_BOX (hbox), props->priv->feature_list, TRUE, TRUE, 0);	
 }
 
 static void
