@@ -227,15 +227,15 @@ gnome_db_custom_layout_new (GnomeDbDict *dict)
 	GnomeDbCustomLayout *gnome_db_custom_layout;
 	guint id;
 
-	g_return_val_if_fail (dict && IS_GNOME_DB_DICT (dict), NULL);
+	g_return_val_if_fail (!dict || IS_GNOME_DB_DICT (dict), NULL);
 
-	obj = g_object_new (GNOME_DB_CUSTOM_LAYOUT_TYPE, "dict", dict, NULL);
+	obj = g_object_new (GNOME_DB_CUSTOM_LAYOUT_TYPE, "dict", ASSERT_DICT (dict), NULL);
         gnome_db_custom_layout = GNOME_DB_CUSTOM_LAYOUT (obj);
 
-	g_object_get (G_OBJECT (dict), "layout_serial", &id, NULL);
+	g_object_get (G_OBJECT (ASSERT_DICT (dict)), "layout_serial", &id, NULL);
 	gnome_db_base_set_id (GNOME_DB_BASE (obj), id);
 
-	gnome_db_dict_declare_layout (dict, gnome_db_custom_layout);
+	gnome_db_dict_declare_layout (ASSERT_DICT (dict), gnome_db_custom_layout);
 
 	return obj;
 }
