@@ -21,6 +21,7 @@
 #include <string.h>
 #include "gnome-db-data-proxy.h"
 #include "gnome-db-data-model.h"
+#include "gnome-db-parameter.h"
 #include "gnome-db-data-set.h"
 #include "marshal.h"
 
@@ -57,8 +58,9 @@ static gint              data_proxy_iter_n_children (GtkTreeModel *tree_model, G
 static gboolean          data_proxy_iter_nth_child  (GtkTreeModel *tree_model, GtkTreeIter *iter, GtkTreeIter *parent, gint n);
 static gboolean          data_proxy_iter_parent     (GtkTreeModel *tree_model, GtkTreeIter *iter, GtkTreeIter *child);
 
-static gint model_to_iter_numbers (GnomeDbDataProxy *proxy, gint row);
-static gint iter_to_model_numbers (GnomeDbDataProxy *proxy, gint row);
+static gint model_to_iter_numbers  (GnomeDbDataProxy *proxy, gint row);
+static gint iter_to_model_numbers  (GnomeDbDataProxy *proxy, gint row);
+static void re_compute_row_numbers (GnomeDbDataProxy *proxy);
 
 static void nullified_object_cb (GnomeDbBase *obj, GnomeDbDataProxy *proxy);
 #ifdef debug
@@ -128,6 +130,9 @@ typedef struct
 struct _GnomeDbDataProxyPrivate
 {
 	GnomeDbDataModel  *model;
+	guint             *cols_non_modif_attrs; /* array of GnomeDbValueAttribute to proxy cols. attributes */
+	GnomeDbDataSet    *info_data_set; /* used for columns attributes and initial values when a new row is created */
+
 	gint               model_nb_cols; /* = gda_data_model_get_n_columns (model) */
 	gint               model_nb_rows; /* = gda_data_model_get_n_rows (model) */
 	gboolean           autocommit;
@@ -183,6 +188,37 @@ iter_to_model_numbers (GnomeDbDataProxy *proxy, gint row)
 	}
 }
 
+/*
+ * Should be called whenever a row in removed from the GtkTreeModel
+ *
+ * Ensures that the RowModif->iter_row are correct and in sunc with what's in @proxy->priv->iter_rows
+ */
+static void
+re_compute_row_numbers (GnomeDbDataProxy *proxy)
+{
+	GSList *list = proxy->priv->new_rows;
+	RowModif *rm;
+	gint num;
+	gint oldnum, newnum;
+
+	num = g_slist_length (proxy->priv->new_rows) - 1;
+	while (list) {
+		rm = ROW_MODIF (list->data);
+
+		oldnum = rm->iter_row;
+		newnum = proxy->priv->model_nb_rows + num + (proxy->priv->add_null_entry ? 1 : 0);
+		if (oldnum != newnum) {
+			/*g_print ("renumbered %d to %d\n", oldnum, newnum);*/
+			rm->iter_row = newnum;
+			g_hash_table_remove (proxy->priv->iter_rows, GINT_TO_POINTER (oldnum));
+			g_hash_table_insert (proxy->priv->iter_rows, GINT_TO_POINTER (newnum), rm);
+		}
+
+		num--;
+		list = g_slist_next (list);
+	}
+}
+
 /*
  * Free a RowModif structure
  *
@@ -349,8 +385,10 @@ gnome_db_data_proxy_new (GnomeDbDataModel *model)
 {
 	GObject   *obj;
 	GnomeDbDataProxy *proxy;
+	gint col;
 
 	g_return_val_if_fail (model && IS_GNOME_DB_DATA_MODEL (model), NULL);
+
 	obj = g_object_new (GNOME_DB_DATA_PROXY_TYPE, "dict", gnome_db_base_get_dict (GNOME_DB_BASE (model)), NULL);
 	proxy = GNOME_DB_DATA_PROXY (obj);
 
@@ -364,6 +402,19 @@ gnome_db_data_proxy_new (GnomeDbDataModel *model)
 	proxy->priv->model_nb_cols = gda_data_model_get_n_columns (GDA_DATA_MODEL (model));
 	proxy->priv->model_nb_rows = gda_data_model_get_n_rows (GDA_DATA_MODEL (model));
 
+	proxy->priv->info_data_set = gnome_db_data_model_get_new_data_set (model);
+	proxy->priv->cols_non_modif_attrs = g_new0 (guint, proxy->priv->model_nb_cols);
+	for (col = 0; col < proxy->priv->model_nb_cols; col++) {
+		GnomeDbParameter *param;
+		
+		param = gnome_db_data_model_get_param_at_column (proxy->priv->model, proxy->priv->info_data_set, col);
+		proxy->priv->cols_non_modif_attrs[col] = GNOME_DB_VALUE_IS_UNCHANGED;
+		if (! gnome_db_parameter_get_not_null (param))
+			proxy->priv->cols_non_modif_attrs[col] |= GNOME_DB_VALUE_CAN_BE_NULL;
+		if (gnome_db_parameter_get_default_value (param))
+			proxy->priv->cols_non_modif_attrs[col] |= GNOME_DB_VALUE_CAN_BE_DEFAULT;
+	}
+
 	g_signal_connect (G_OBJECT (model), "data-refreshed",
 			  G_CALLBACK (model_data_refreshed_cb), proxy);
 
@@ -389,12 +440,22 @@ gnome_db_data_proxy_dispose (GObject *object)
 	if (proxy->priv) {
 		gnome_db_base_nullify_check (GNOME_DB_BASE (object));
 		
+		if (proxy->priv->info_data_set) {
+			g_object_unref (proxy->priv->info_data_set);
+			proxy->priv->info_data_set = NULL;
+		}
+
 		if (proxy->priv->model) {
 			g_signal_handlers_disconnect_by_func (G_OBJECT (proxy->priv->model),
                                                               G_CALLBACK (nullified_object_cb), proxy);
 			g_object_unref (proxy->priv->model);
 			proxy->priv->model = NULL;
 		}
+
+		if (proxy->priv->cols_non_modif_attrs) {
+			g_free (proxy->priv->cols_non_modif_attrs);
+			proxy->priv->cols_non_modif_attrs = NULL;
+		}
 		
 		if (proxy->priv->iter_rows) {
 			gnome_db_data_proxy_reset_all (proxy);
@@ -432,10 +493,10 @@ gnome_db_data_proxy_finalize (GObject   * object)
 
 
 static void 
-gnome_db_data_proxy_set_property    (GObject              *object,
-				    guint                 param_id,
-				    const GValue         *value,
-				    GParamSpec           *pspec)
+gnome_db_data_proxy_set_property (GObject              *object,
+				  guint                 param_id,
+				  const GValue         *value,
+				  GParamSpec           *pspec)
 {
 	GnomeDbDataProxy *proxy;
 
@@ -485,10 +546,10 @@ gnome_db_data_proxy_set_property    (GObject              *object,
 }
 
 static void
-gnome_db_data_proxy_get_property    (GObject              *object,
-				    guint                 param_id,
-				    GValue               *value,
-				    GParamSpec           *pspec)
+gnome_db_data_proxy_get_property (GObject              *object,
+				  guint                 param_id,
+				  GValue               *value,
+				  GParamSpec           *pspec)
 {
 	GnomeDbDataProxy *proxy;
 
@@ -526,7 +587,6 @@ model_data_refreshed_cb (GnomeDbDataModel *model, GnomeDbDataProxy *proxy)
 		gint i, old_nb_rows;
 		GtkTreePath *path;
 		GtkTreeIter iter;
-
 		old_nb_rows = proxy->priv->model_nb_rows;
 
 		gnome_db_data_proxy_reset_all (proxy);
@@ -534,7 +594,7 @@ model_data_refreshed_cb (GnomeDbDataModel *model, GnomeDbDataProxy *proxy)
 		proxy->priv->model_nb_rows = gda_data_model_get_n_rows (GDA_DATA_MODEL (model));
 		proxy->priv->model_refresh_pending = FALSE;
 
-		/* emit the GtkTreeModel::"row_changed" signal for all the previous rows */
+		/* emit the GtkTreeModel::"row_changed" signal for all the rows which already existed */
 		for (i=0; (i < old_nb_rows) && (i < proxy->priv->model_nb_rows); i++) {
 			path = gtk_tree_path_new_from_indices (model_to_iter_numbers (proxy, i), -1);
 			data_proxy_get_iter (GTK_TREE_MODEL (proxy), &iter, path);
@@ -550,14 +610,16 @@ model_data_refreshed_cb (GnomeDbDataModel *model, GnomeDbDataProxy *proxy)
 				gtk_tree_model_row_inserted (GTK_TREE_MODEL (proxy), path, &iter);
 				gtk_tree_path_free (path);
 			}
+			proxy->priv->stamp = g_random_int ();
 		}
 		else {
-			/* emit the GtkTreeModel::"row_deleted" signal for all the new rows */
-			for (; i < old_nb_rows; i++) {
-				path = gtk_tree_path_new_from_indices (model_to_iter_numbers (proxy, i), -1);
+			/* emit the GtkTreeModel::"row_deleted" signal for all the new rows 
+			 * (using the same row number!) */
+			path = gtk_tree_path_new_from_indices (model_to_iter_numbers (proxy, i), -1);
+			for (; i < old_nb_rows; i++) 
 				gtk_tree_model_row_deleted (GTK_TREE_MODEL (proxy), path);
-				gtk_tree_path_free (path);
-			}	
+			gtk_tree_path_free (path);
+			proxy->priv->stamp = g_random_int ();
 		}
 	}
 }
@@ -617,6 +679,9 @@ gnome_db_data_proxy_is_read_only (GnomeDbDataProxy *proxy)
 	return ! gda_data_model_is_updatable (GDA_DATA_MODEL (proxy->priv->model));
 }
 
+
+RowModif *find_or_create_row_modif (GnomeDbDataProxy *proxy, GtkTreeIter *iter, gint col, RowValue **ret_rv);
+
 /**
  * gnome_db_data_proxy_set_value
  * @proxy: a #GnomeDbDataProxy object
@@ -683,29 +748,7 @@ gnome_db_data_proxy_set_value (GnomeDbDataProxy *proxy, GtkTreeIter *iter,
 		g_free (cmp_value);
 
 		/* from now on we have a new value for the row */
-		rm = g_hash_table_lookup (proxy->priv->iter_rows, iter->user_data);
-		if (!rm) {
-			/* create a new RowModif */
-			rm = row_modifs_new (proxy, iter);
-			rm->model_row = iter_to_model_numbers (proxy, row);
-			rm->iter_row = row;
-			g_hash_table_insert (proxy->priv->iter_rows, GINT_TO_POINTER (row), rm);
-			proxy->priv->all_modifs = g_slist_prepend (proxy->priv->all_modifs, rm);
-			
-			rv = NULL;
-			/* g_print ("New RM\n"); */
-		}
-		else {
-			/* there are already some modifications to the row, try to catch the RowValue if available */
-			GSList *list;
-			
-			list = rm->modif_values;
-			while (list && !rv) {
-				if (ROW_VALUE (list->data)->model_column == col)
-					rv = ROW_VALUE (list->data);
-				list = g_slist_next (list);
-			}
-		}
+		rm = find_or_create_row_modif (proxy, iter, col, &rv);
 
 		if (rv) {
 			/* compare with the original value (before modifications) and either delete the RowValue, 
@@ -722,11 +765,16 @@ gnome_db_data_proxy_set_value (GnomeDbDataProxy *proxy, GtkTreeIter *iter,
 				/* remove the RowValue */
 				rm->modif_values = g_slist_remove (rm->modif_values, rv);
 				g_free (rv);
+				rv = NULL;
 			}
 			else {
 				/* simply alter the RowValue */
-				if (value)
+				if (value) {
+					rv->attributes &= ~GNOME_DB_VALUE_IS_NULL;
 					rv->value = gda_value_copy (value);
+				}
+				else
+					rv->attributes |= GNOME_DB_VALUE_IS_NULL;
 			}
 		}
 		else {
@@ -735,12 +783,23 @@ gnome_db_data_proxy_set_value (GnomeDbDataProxy *proxy, GtkTreeIter *iter,
 			rv = g_new0 (RowValue, 1);
 			rv->row_modif = rm;
 			rv->model_column = col;
+			rv->attributes = proxy->priv->cols_non_modif_attrs [col];
+
 			if (value && !gda_value_is_null (value)) 
 				rv->value = gda_value_copy (value);
 			else
-				rv->attributes = GNOME_DB_VALUE_IS_NULL;
+				rv->attributes |= GNOME_DB_VALUE_IS_NULL;
+			if (rm->model_row >= 0)
+				rv->attributes |= GNOME_DB_VALUE_HAS_VALUE_ORIG;
+			else
+				rv->attributes &= ~GNOME_DB_VALUE_HAS_VALUE_ORIG;
 			rm->modif_values = g_slist_prepend (rm->modif_values, rv);
 		}
+		
+		if (rv) {
+			rv->attributes &= ~GNOME_DB_VALUE_IS_UNCHANGED;
+			rv->attributes &= ~GNOME_DB_VALUE_IS_DEFAULT;
+		}
 
 		if (!rm->to_be_deleted && !rm->modif_values && (rm->model_row >= 0)) {
 			/* remove that RowModif, it's useless */
@@ -753,8 +812,67 @@ gnome_db_data_proxy_set_value (GnomeDbDataProxy *proxy, GtkTreeIter *iter,
 		gtk_tree_model_row_changed (GTK_TREE_MODEL (proxy), path, iter);
 	}
 
-	if ((col >= proxy->priv->model_nb_cols) && (col < 2*proxy->priv->model_nb_cols))
-		g_warning ("Trying to set read-only column: %d", col);
+	if ((col >= proxy->priv->model_nb_cols) && (col < 2*proxy->priv->model_nb_cols)) {
+		/* Storing an action => apply that action */
+		guint action;
+		gint model_col = col - proxy->priv->model_nb_cols;
+		
+		if (gda_value_isa (value, GDA_VALUE_TYPE_UINTEGER)) {
+			gboolean action_ok = FALSE;
+			
+			action = gda_value_get_uinteger (value);
+			
+			if ((action & GNOME_DB_VALUE_IS_NULL) ||
+			    (action & GNOME_DB_VALUE_IS_DEFAULT) ||
+			    (action & GNOME_DB_VALUE_IS_UNCHANGED)) 
+				action_ok = TRUE;
+			if (action_ok) {
+				if (action & GNOME_DB_VALUE_IS_NULL) 
+					gnome_db_data_proxy_set_value (proxy, iter, model_col, NULL);
+				else {
+					RowModif *rm;
+					RowValue *rv = NULL;
+
+					rm = find_or_create_row_modif (proxy, iter, model_col, &rv);
+					g_assert (rm);
+
+					if (action & GNOME_DB_VALUE_IS_DEFAULT) {
+						if (!rv) {
+							/* create a new RowValue */
+							/*g_print ("New RV\n");*/
+							rv = g_new0 (RowValue, 1);
+							rv->row_modif = rm;
+							rv->model_column = model_col;
+							rv->attributes = proxy->priv->cols_non_modif_attrs [col];
+							
+							rv->value = NULL;
+							rv->attributes &= ~GNOME_DB_VALUE_IS_UNCHANGED;
+							if (rm->model_row >= 0)
+								rv->attributes |= GNOME_DB_VALUE_HAS_VALUE_ORIG;
+							else
+								rv->attributes &= ~GNOME_DB_VALUE_HAS_VALUE_ORIG;
+							
+							rm->modif_values = g_slist_prepend (rm->modif_values, rv);	
+						}
+
+						rv->attributes |= GNOME_DB_VALUE_IS_DEFAULT;
+						gtk_tree_model_row_changed (GTK_TREE_MODEL (proxy), path, iter);
+					}
+					if (action & GNOME_DB_VALUE_IS_UNCHANGED) {
+						if (!rm->orig_values)
+							g_warning ("Action = GNOME_DB_VALUE_IS_UNCHANGED, no RowValue!");
+						else
+							gnome_db_data_proxy_set_value (proxy, iter, model_col,
+										       rm->orig_values [model_col]);
+					}
+				}
+			}
+			else
+				g_warning ("Trying to set read-only column: %d", col);
+		}
+		else 
+			g_warning ("Column %d needs a GDA_VALUE_TYPE_UINTEGER value", col);	
+	}
 
 	gtk_tree_path_free (path);
 
@@ -763,9 +881,52 @@ gnome_db_data_proxy_set_value (GnomeDbDataProxy *proxy, GtkTreeIter *iter,
 		return FALSE;
 	}
 
+#ifdef debug_NO
+	gnome_db_base_dump (proxy, 5);
+#endif
+
 	return TRUE;
 }
 
+/*
+ * Stores the new RowValue in @rv
+ */
+RowModif *
+find_or_create_row_modif (GnomeDbDataProxy *proxy, GtkTreeIter *iter, gint col, RowValue **ret_rv)
+{
+	RowModif *rm;
+	RowValue *rv = NULL;
+	gint row;
+
+	row = GPOINTER_TO_INT (iter->user_data);
+	rm = g_hash_table_lookup (proxy->priv->iter_rows, iter->user_data);
+	if (!rm) {
+		/* create a new RowModif */
+		rm = row_modifs_new (proxy, iter);
+		rm->model_row = iter_to_model_numbers (proxy, row);
+		rm->iter_row = row;
+		g_hash_table_insert (proxy->priv->iter_rows, GINT_TO_POINTER (row), rm);
+		proxy->priv->all_modifs = g_slist_prepend (proxy->priv->all_modifs, rm);
+		
+		/* g_print ("New RM\n"); */	
+	}
+	else {
+		/* there are already some modifications to the row, try to catch the RowValue if available */
+		GSList *list;
+		
+		list = rm->modif_values;
+		while (list && !rv) {
+			if (ROW_VALUE (list->data)->model_column == col)
+				rv = ROW_VALUE (list->data);
+			list = g_slist_next (list);
+		}
+	}
+
+	if (ret_rv)
+		*ret_rv = rv;
+	return rm;
+}
+
 /**
  * gnome_db_data_proxy_get_value
  * @proxy: a #GnomeDbDataProxy object
@@ -859,15 +1020,16 @@ gnome_db_data_proxy_delete (GnomeDbDataProxy *proxy, GtkTreeIter *iter)
 	if (rm) {
 		if (! rm->to_be_deleted) {
 			if (rm->model_row < 0) {
-				/* remove the row completely because it was does not exist in the data model */
+				/* remove the row completely because it does not exist in the data model */
 				GtkTreePath *path;
 				
 				g_hash_table_remove (proxy->priv->iter_rows, GINT_TO_POINTER (rm->iter_row));
 				proxy->priv->all_modifs = g_slist_remove (proxy->priv->all_modifs, rm);
 				proxy->priv->new_rows = g_slist_remove (proxy->priv->new_rows, rm);
 				row_modifs_free (rm);
-				proxy->priv->stamp = g_random_int ();
-				
+				re_compute_row_numbers (proxy);
+
+				proxy->priv->stamp = g_random_int ();				
 				path = gtk_tree_path_new ();
 				gtk_tree_path_append_index (path, GPOINTER_TO_INT (iter->user_data));
 				gtk_tree_model_row_deleted (GTK_TREE_MODEL (proxy), path);
@@ -1036,7 +1198,10 @@ gnome_db_data_proxy_append (GnomeDbDataProxy *proxy, GtkTreeIter *iter)
 {
 	RowModif *rm;
 	GtkTreePath *path;
-
+	gint col;
+	GdaValue *value;
+	const GdaValue *cvalue;
+	
 	g_return_if_fail (proxy && IS_GNOME_DB_DATA_PROXY (proxy));
 	g_return_if_fail (proxy->priv);
 	g_return_if_fail (iter);
@@ -1047,9 +1212,6 @@ gnome_db_data_proxy_append (GnomeDbDataProxy *proxy, GtkTreeIter *iter)
 	rm->orig_values_size = 0;
 	rm->iter_row = gnome_db_data_proxy_get_n_rows (proxy);
 
-	/* FIXME: for values which will be invalid for this row, set them to the be the default value */
-	TO_IMPLEMENT;
-
 	proxy->priv->all_modifs = g_slist_prepend (proxy->priv->all_modifs, rm);
 	proxy->priv->new_rows = g_slist_prepend (proxy->priv->new_rows, rm);
 	g_hash_table_insert (proxy->priv->iter_rows, GINT_TO_POINTER (rm->iter_row), rm);
@@ -1059,6 +1221,24 @@ gnome_db_data_proxy_append (GnomeDbDataProxy *proxy, GtkTreeIter *iter)
 	data_proxy_get_iter (GTK_TREE_MODEL (proxy), iter, path);
 	gtk_tree_model_row_inserted (GTK_TREE_MODEL (proxy), path, iter);
 	gtk_tree_path_free (path);
+
+	/* for the columns which allow a default value, set them to the default value */
+	value = gda_value_new_uinteger (GNOME_DB_VALUE_IS_DEFAULT);
+	for (col = 0; col < proxy->priv->model_nb_cols; col ++) {
+		GnomeDbParameter *param;
+		
+		/* columns attributes */
+		if (proxy->priv->cols_non_modif_attrs [col] & GNOME_DB_VALUE_CAN_BE_DEFAULT)
+			gnome_db_data_proxy_set_value (proxy, iter, proxy->priv->model_nb_cols + col, value);
+
+		/* columns original values */
+		param = gnome_db_data_model_get_param_at_column (proxy->priv->model, proxy->priv->info_data_set, col);
+		if (gnome_db_parameter_is_valid (param)) {
+			cvalue = gnome_db_parameter_get_value (param);
+			gnome_db_data_proxy_set_value (proxy, iter, col, cvalue);
+		}
+	}
+	gda_value_free (value);
 }
 
 /**
@@ -1093,38 +1273,36 @@ gnome_db_data_proxy_reset_all (GnomeDbDataProxy *proxy)
 	GtkTreePath *path;
 	GtkTreeIter iter;
 
-	/* new rows are first treated and removed */
+	/* new rows are first treated and removed (no memory de-allocation here) */
 	if (proxy->priv->new_rows) {
-		GSList *list = proxy->priv->new_rows;
-		proxy->priv->stamp = g_random_int ();
-
 		while (proxy->priv->new_rows) {
 			path = gtk_tree_path_new ();
-			gtk_tree_path_append_index (path, ROW_MODIF (list->data)->iter_row);
+			gtk_tree_path_append_index (path, ROW_MODIF (proxy->priv->new_rows->data)->iter_row);
+			proxy->priv->new_rows = g_slist_delete_link (proxy->priv->new_rows, proxy->priv->new_rows);
+			re_compute_row_numbers (proxy);
+
 			gtk_tree_model_row_deleted (GTK_TREE_MODEL (proxy), path);
 			gtk_tree_path_free (path);
-
-			proxy->priv->new_rows = g_slist_delete_link (proxy->priv->new_rows, proxy->priv->new_rows);
 		}
+
+		proxy->priv->stamp = g_random_int ();
 	}
 
-	/* other modified rows are then treated */
-	if (proxy->priv->all_modifs) {
-		while (proxy->priv->all_modifs) {
-			gint iter_row = ROW_MODIF (proxy->priv->all_modifs->data)->iter_row;
-			gint model_row = ROW_MODIF (proxy->priv->all_modifs->data)->model_row;
-
-			row_modifs_free (ROW_MODIF (proxy->priv->all_modifs->data));
-			g_hash_table_remove (proxy->priv->iter_rows, GINT_TO_POINTER (iter_row));
-			proxy->priv->all_modifs = g_slist_delete_link (proxy->priv->all_modifs, proxy->priv->all_modifs);
-			
-			if (model_row >= 0) {
-				/* not a new row */
-				path = gtk_tree_path_new_from_indices (iter_row, -1);
-				data_proxy_get_iter (GTK_TREE_MODEL (proxy), &iter, path);
-				gtk_tree_model_row_changed (GTK_TREE_MODEL (proxy), path, &iter);
-				gtk_tree_path_free (path);
-			}
+	/* all modified rows are then treated (including memory de-allocation for new rows) */
+	while (proxy->priv->all_modifs) {
+		gint iter_row = ROW_MODIF (proxy->priv->all_modifs->data)->iter_row;
+		gint model_row = ROW_MODIF (proxy->priv->all_modifs->data)->model_row;
+		
+		row_modifs_free (ROW_MODIF (proxy->priv->all_modifs->data));
+		g_hash_table_remove (proxy->priv->iter_rows, GINT_TO_POINTER (iter_row));
+		proxy->priv->all_modifs = g_slist_delete_link (proxy->priv->all_modifs, proxy->priv->all_modifs);
+		
+		if (model_row >= 0) {
+			/* not a new row */
+			path = gtk_tree_path_new_from_indices (iter_row, -1);
+			data_proxy_get_iter (GTK_TREE_MODEL (proxy), &iter, path);
+			gtk_tree_model_row_changed (GTK_TREE_MODEL (proxy), path, &iter);
+			gtk_tree_path_free (path);
 		}
 	}
 }
@@ -1173,13 +1351,16 @@ gnome_db_data_proxy_reset_value (GnomeDbDataProxy *proxy, GtkTreeIter *iter, gin
 				rm->modif_values = g_slist_remove (rm->modif_values, rv);
 				if (!rm->modif_values && !rm->to_be_deleted) {
 					/* remove this RowList as well */
-					gtk_tree_model_row_deleted (GTK_TREE_MODEL (proxy), path);
 					g_hash_table_remove (proxy->priv->iter_rows, GINT_TO_POINTER (rm->iter_row));
-					proxy->priv->stamp = g_random_int ();
 					proxy->priv->all_modifs = g_slist_remove (proxy->priv->all_modifs, rm);
-					if (rm->model_row < 0)
+					if (rm->model_row < 0) {
 						proxy->priv->new_rows = g_slist_remove (proxy->priv->new_rows, rm);
+						re_compute_row_numbers (proxy);
+					}
 					row_modifs_free (rm);
+
+					proxy->priv->stamp = g_random_int ();
+					gtk_tree_model_row_deleted (GTK_TREE_MODEL (proxy), path);
 				}
 				else
 					gtk_tree_model_row_changed (GTK_TREE_MODEL (proxy), path, iter);	
@@ -1223,7 +1404,7 @@ gnome_db_data_proxy_commit_all (GnomeDbDataProxy *proxy, GError **error)
 				allok = FALSE;
 	}
 
-	gda_data_model_thaw (GDA_DATA_MODEL (proxy->priv->model));
+	gda_data_model_thaw (GDA_DATA_MODEL (proxy->priv->model)); /* => might make a refresh */
 
 	return allok;
 }
@@ -1250,6 +1431,11 @@ gnome_db_data_proxy_commit_row (GnomeDbDataProxy *proxy, GtkTreeIter *iter, GErr
 	g_return_val_if_fail (iter, FALSE);
 	g_return_val_if_fail (iter->stamp == proxy->priv->stamp, FALSE);
 
+#ifdef debug_NO
+	DEBUG_HEADER;
+	gnome_db_base_dump (proxy, 5);
+#endif
+
 	row = GPOINTER_TO_INT (iter->user_data);
 	rm = g_hash_table_lookup (proxy->priv->iter_rows, iter->user_data);
 	if (rm) {
@@ -1271,9 +1457,11 @@ gnome_db_data_proxy_commit_row (GnomeDbDataProxy *proxy, GtkTreeIter *iter, GErr
 				gint i;
 				gboolean newvalue_found;
 				GdaValue *newvalue;
+				gboolean *is_default;
 				
 				g_assert (rm->modif_values);
 				g_assert (rm->orig_values);
+				is_default = g_new0 (gboolean, proxy->priv->model_nb_cols);
 				for (i=0; i < rm->orig_values_size; i++) {
 					newvalue_found = FALSE;
 					newvalue = NULL;
@@ -1283,6 +1471,8 @@ gnome_db_data_proxy_commit_row (GnomeDbDataProxy *proxy, GtkTreeIter *iter, GErr
 						if (ROW_VALUE (list->data)->model_column == i) {
 							newvalue = ROW_VALUE (list->data)->value;
 							newvalue_found = TRUE;
+							is_default [i] = ROW_VALUE (list->data)->attributes & 
+								GNOME_DB_VALUE_IS_DEFAULT;
 						}
 						list = g_slist_next (list);
 					}
@@ -1293,8 +1483,13 @@ gnome_db_data_proxy_commit_row (GnomeDbDataProxy *proxy, GtkTreeIter *iter, GErr
 				gdarow = gda_row_new_from_list (GDA_DATA_MODEL (proxy->priv->model), values);
 				g_list_free (values);
 				gda_row_set_number (gdarow, rm->model_row);
+				for (i = 0; i < proxy->priv->model_nb_cols; i++) {
+					if (is_default [i])
+						gda_row_set_is_default (gdarow, i, TRUE);
+				}
 				err = ! gda_data_model_update_row (GDA_DATA_MODEL (proxy->priv->model), gdarow);
 				gda_row_free (gdarow);
+				g_free (is_default);
 			}
 			else {
 				/* insert a new row */
@@ -1303,15 +1498,20 @@ gnome_db_data_proxy_commit_row (GnomeDbDataProxy *proxy, GtkTreeIter *iter, GErr
 				GList *values = NULL;
 				gint i;
 				GdaValue *newvalue;
+				gboolean *is_default;
 				
 				g_assert (rm->modif_values);
-				for (i=0; i < proxy->priv->model_nb_cols; i++) {
+				is_default = g_new0 (gboolean, proxy->priv->model_nb_cols);
+				for (i = 0; i < proxy->priv->model_nb_cols; i++) {
 					newvalue = NULL;
 					
 					list = rm->modif_values;
 					while (list && !newvalue) {
-						if (ROW_VALUE (list->data)->model_column == i)
+						if (ROW_VALUE (list->data)->model_column == i) {
 							newvalue = ROW_VALUE (list->data)->value;
+							is_default [i] = ROW_VALUE (list->data)->attributes & 
+								GNOME_DB_VALUE_IS_DEFAULT;
+						}
 						list = g_slist_next (list);
 					}
 					values = g_list_append (values, newvalue);
@@ -1319,18 +1519,20 @@ gnome_db_data_proxy_commit_row (GnomeDbDataProxy *proxy, GtkTreeIter *iter, GErr
 				
 				gdarow = gda_row_new_from_list (GDA_DATA_MODEL (proxy->priv->model), values);
 				g_list_free (values);
+				for (i = 0; i < proxy->priv->model_nb_cols; i++) {
+					if (is_default [i]) {
+						gda_row_set_is_default (gdarow, i, TRUE);
+					}
+				}
 				err = ! gda_data_model_append_row (GDA_DATA_MODEL (proxy->priv->model), gdarow);
-				gda_row_free (gdarow);				
+				gda_row_free (gdarow);
+				g_free (is_default);
 			}
 		}
 		proxy->priv->internal_changes --;
 
 		if (!err) {
-			/* if no error occured, then we can remove that RowModif */
-			g_hash_table_remove (proxy->priv->iter_rows, GINT_TO_POINTER (rm->iter_row));
-			proxy->priv->all_modifs = g_slist_remove (proxy->priv->all_modifs, rm);
-			proxy->priv->new_rows = g_slist_remove (proxy->priv->new_rows, rm);
-			row_modifs_free (rm);
+			/* data refresh */
 			if (proxy->priv->model_refresh_pending)
 				model_data_refreshed_cb (proxy->priv->model, proxy);
 		}
@@ -1512,7 +1714,7 @@ data_proxy_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, gint column,
 									    2 <=> original value */
 
 			rm = g_hash_table_lookup (proxy->priv->iter_rows, iter->user_data);
-			if (what == 2) {
+			if (what == 2) { /* original value */
 				if (rm) {
 					if (rm->orig_values)
 						g_value_set_pointer (value, 
@@ -1523,8 +1725,8 @@ data_proxy_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, gint column,
 				else {
 					gint model_row = iter_to_model_numbers (proxy, row);
 					g_value_set_pointer (value,
-							   gda_data_model_get_value_at (GDA_DATA_MODEL (proxy->priv->model), 
-											model_col, model_row));
+							     gda_data_model_get_value_at (GDA_DATA_MODEL (proxy->priv->model), 
+											  model_col, model_row));
 					
 				}
 				value_set = TRUE;
@@ -1537,7 +1739,7 @@ data_proxy_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, gint column,
 					
 					list = rm->modif_values;
 					while (list && !rv) {
-						if (ROW_VALUE (list->data)->model_column == column)
+						if (ROW_VALUE (list->data)->model_column == model_col)
 							rv = ROW_VALUE (list->data);
 						list = g_slist_next (list);
 					}
@@ -1545,7 +1747,7 @@ data_proxy_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, gint column,
 						value_has_modifs = TRUE;
 						if (what == 0)
 							g_value_set_pointer (value, rv->value);
-						else
+						else 
 							g_value_set_uint (value, rv->attributes);
 						value_set = TRUE;
 					}
@@ -1559,18 +1761,32 @@ data_proxy_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, gint column,
 						/* existing row */
 						if (column < proxy->priv->model_nb_cols) 
 							g_value_set_pointer (value, 
-							   gda_data_model_get_value_at (GDA_DATA_MODEL (proxy->priv->model), 
-											column, model_row));
-						else
-							g_value_set_uint (value, GNOME_DB_VALUE_IS_UNCHANGED);
+							gda_data_model_get_value_at (GDA_DATA_MODEL (proxy->priv->model), 
+										     column, model_row));
+						else {
+							const GdaValue *gdavalue;
+							guint retval;
+
+							gdavalue = gda_data_model_get_value_at 
+								(GDA_DATA_MODEL (proxy->priv->model), model_col, model_row);
+							retval = proxy->priv->cols_non_modif_attrs [model_col];
+							if (!gdavalue || gda_value_is_null (gdavalue))
+								retval |= GNOME_DB_VALUE_IS_NULL;
+							
+							g_value_set_uint (value, retval);
+						}
 					}
 					else {
 						/* new row */
 						if (column < proxy->priv->model_nb_cols) 
 							g_value_set_pointer (value, NULL);
-						else
-							g_value_set_uint (value, GNOME_DB_VALUE_IS_UNCHANGED | 
-									  GNOME_DB_VALUE_IS_NULL);	
+						else {
+							guint retval;
+
+							retval = proxy->priv->cols_non_modif_attrs [model_col];
+							retval |= GNOME_DB_VALUE_IS_NULL;
+							g_value_set_uint (value, retval);
+						}
 					}
 				}
 			}
@@ -1595,7 +1811,7 @@ data_proxy_iter_next (GtkTreeModel *tree_model, GtkTreeIter *iter)
 	
 	row = GPOINTER_TO_INT (iter->user_data);
 	row++;
-	if (row >= gnome_db_data_proxy_get_n_rows (proxy))
+	if (row >= gnome_db_data_proxy_get_n_rows (proxy)) 
 		return FALSE;
 	else {
 		iter->user_data = GINT_TO_POINTER (row);
@@ -1696,7 +1912,10 @@ gnome_db_data_proxy_dump (GnomeDbDataProxy *proxy, guint offset)
 	
 	for (i = 0; i < n_cols; i++) {
 		str = gda_data_model_get_column_title (GDA_DATA_MODEL (proxy->priv->model), i);
-		cols_size [i] = strlen (str);
+		if (str && *str)
+			cols_size [i] = strlen (str);
+		else
+			cols_size [i] = 0;	
 	}
 
 	/* ... and using column data */
