@@ -192,9 +192,9 @@ gnome_db_data_set_new (GnomeDbDict *dict, GSList *params)
 	GObject *obj;
 	GnomeDbDataSet *gnome_db_data_set;
 
-	g_return_val_if_fail (dict && IS_GNOME_DB_DICT (dict), NULL);
+	g_return_val_if_fail (!dict || IS_GNOME_DB_DICT (dict), NULL);
 
-	obj = g_object_new (GNOME_DB_DATA_SET_TYPE, "dict", dict, NULL);
+	obj = g_object_new (GNOME_DB_DATA_SET_TYPE, "dict", ASSERT_DICT (dict), NULL);
         gnome_db_data_set = GNOME_DB_DATA_SET (obj);
         gnome_db_base_set_id (GNOME_DB_BASE (gnome_db_data_set), 0);
 
@@ -546,7 +546,7 @@ compute_dataset_nodes (GnomeDbDataSet *dataset)
 						rs = GNOME_DB_RESULT_SET (gnome_db_result_set_new (GNOME_DB_QUERY (ent), 
 												   NULL));
 						gnome_db_base_set_name (GNOME_DB_BASE (rs),
-									gnome_db_base_get_name (GNOME_DB_BASE (ent)));
+									gnome_db_base_get_name (GNOME_DB_BASE (list->data)));
 						g_hash_table_insert (new_rs, ent, rs);
 						g_object_get (G_OBJECT (rs), "init_query_replacements", &repl, NULL);
 					}
