@@ -390,12 +390,12 @@ gnome_db_database_new (GnomeDbDict *dict)
 	GObject   *obj;
 	GnomeDbDatabase *gnome_db_database;
 
-	g_return_val_if_fail (dict && IS_GNOME_DB_DICT (dict), NULL);
+	g_return_val_if_fail (!dict || IS_GNOME_DB_DICT (dict), NULL);
 
 	obj = g_object_new (GNOME_DB_DATABASE_TYPE, NULL);
 	gnome_db_database = GNOME_DB_DATABASE (obj);
 
-	gnome_db_base_set_dict (GNOME_DB_BASE (gnome_db_database), dict);
+	gnome_db_base_set_dict (GNOME_DB_BASE (gnome_db_database), ASSERT_DICT (dict));
 
 	return obj;
 }
