@@ -1,6 +1,6 @@
 /* gnome-db-dbms-update-viewer.c
  *
- * Copyright (C) 2002 Vivien Malerba
+ * Copyright (C) 2002 - 2005 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -103,12 +103,12 @@ gnome_db_dbms_update_viewer_new (GnomeDbDict *dict)
 	GObject    *obj;
 	GnomeDbmsUpdateViewer *mgv;
 
-	g_return_val_if_fail (dict && IS_GNOME_DB_DICT (dict), NULL);
+	g_return_val_if_fail (!dict || IS_GNOME_DB_DICT (dict), NULL);
 
 	obj = g_object_new (GNOME_DB_DBMS_UPDATE_VIEWER_TYPE, NULL);
 	mgv = GNOME_DB_DBMS_UPDATE_VIEWER (obj);
 
-	mgv->priv->dict = dict;
+	mgv->priv->dict = ASSERT_DICT (dict);
 
 	g_object_weak_ref (G_OBJECT (mgv->priv->dict),
 			   (GWeakNotify) gnome_db_dict_weak_notify, mgv);
