@@ -264,8 +264,8 @@ gnome_db_dsn_config_set_info (GnomeDbDsnConfig *config, const GdaDataSourceInfo
 	gtk_entry_set_text (GTK_ENTRY (config->priv->wname), dsn_info->name);
 	gnome_db_option_menu_set_selection (GTK_OPTION_MENU (config->priv->wprovider),
 					    dsn_info->provider);
-	gtk_entry_set_text (GTK_ENTRY (config->priv->wstring), dsn_info->cnc_string);
-	gtk_entry_set_text (GTK_ENTRY (config->priv->wdesc), dsn_info->description);
-	gtk_entry_set_text (GTK_ENTRY (config->priv->wuser), dsn_info->username);
-	gtk_entry_set_text (GTK_ENTRY (config->priv->wpassword), dsn_info->password);
+	gtk_entry_set_text (GTK_ENTRY (config->priv->wstring), dsn_info->cnc_string ? dsn_info->cnc_string : "");
+	gtk_entry_set_text (GTK_ENTRY (config->priv->wdesc), dsn_info->description ? dsn_info->description : "");
+	gtk_entry_set_text (GTK_ENTRY (config->priv->wuser), dsn_info->username ? dsn_info->username : "");
+	gtk_entry_set_text (GTK_ENTRY (config->priv->wpassword), dsn_info->password ? dsn_info->password : "");
 }
