@@ -86,6 +86,11 @@ static GtkActionGroup *gnome_db_grid_get_actions_group     (GnomeDbDataWidget *i
 static GnomeDbDataSet *gnome_db_grid_widget_get_params_set (GnomeDbDataWidget *iface);
 static GnomeDbDataSet *gnome_db_grid_widget_get_data_set   (GnomeDbDataWidget *iface);
 
+static GdaDataModel   *gnome_db_grid_widget_get_gda_model             (GnomeDbDataWidget *iface);
+static void            gnome_db_grid_widget_set_gda_model             (GnomeDbDataWidget *iface, GnomeDbDict *dict, GdaDataModel *model);
+static void            gnome_db_grid_widget_set_title                 (GnomeDbDataWidget *iface, const gchar *title);
+static void            gnome_db_grid_widget_set_show_title            (GnomeDbDataWidget *iface, gboolean show);
+static void            gnome_db_grid_widget_set_title_icon_from_stock (GnomeDbDataWidget *iface, const gchar *stock_id, GtkIconSize size);
 
 typedef struct {
 	GnomeDbDataSetNode *node;
@@ -104,6 +109,12 @@ struct _GnomeDbGridPriv
 	GnomeDbDataModel  *data_model;
 	GnomeDbDataSet    *data_set;
 
+	GSList            *nodes_infos; /* list of DataSetNodeInfo structures */
+
+	GHashTable        *nodes_gtk_row_refs; /* key = node, value = list of GtkTreeRowReference holding list of values for that node */
+	GHashTable        *row_ref_values;     /* key = GtkTreeRowReference, value = list of values for that key */
+	GSList            *gtk_tree_row_references; /* list of GtkTreeRowReference used as key in 'row_ref_values' */
+
 	guint              mode;
 	GtkTooltips       *tooltips;
 	GSList            *columns_data; /* list of ColumnData */
@@ -195,6 +206,11 @@ gnome_db_grid_widget_init (GnomeDbDataWidgetIface *iface)
 	iface->get_actions_group = gnome_db_grid_get_actions_group;
 	iface->get_params_set = gnome_db_grid_widget_get_params_set;
 	iface->get_data_set = gnome_db_grid_widget_get_data_set;
+	iface->get_gda_model = gnome_db_grid_widget_get_gda_model;
+	iface->set_gda_model = gnome_db_grid_widget_set_gda_model;
+	iface->set_show_title = gnome_db_grid_widget_set_show_title;
+	iface->set_title = gnome_db_grid_widget_set_title;
+	iface->set_t_i_from_stock = gnome_db_grid_widget_set_title_icon_from_stock;
 }
 
 static void
@@ -253,7 +269,7 @@ gnome_db_grid_init (GnomeDbGrid * wid)
 {
 	wid->priv = g_new0 (GnomeDbGridPriv, 1);
 	wid->priv->proxy = NULL;
-	wid->priv->default_show_info_cell = FALSE;
+	wid->priv->default_show_info_cell = TRUE;
 	wid->priv->default_show_global_actions = TRUE;
 	wid->priv->default_column_editable = TRUE;
 
@@ -268,6 +284,10 @@ gnome_db_grid_init (GnomeDbGrid * wid)
 
 	wid->priv->mode = 0;
 	wid->priv->tooltips = NULL;
+
+	/* title */
+	wid->priv->title = gnome_db_gray_bar_new (_("No title"));
+        gtk_box_pack_start (GTK_BOX (wid), wid->priv->title, FALSE, TRUE, 2);
 }
 
 /**
@@ -289,7 +309,7 @@ gnome_db_grid_new (void)
 
 /**
  * gnome_db_grid_new_with_gda_model
- * @dict: a #GnomeDbDict dictionnary, or %NULL
+ * @dict: a #GnomeDbDict dictionary, or %NULL
  * @model: a #GdaDataModel
  *
  * Creates a new #GnomeDbGrid widget suitable to display the data in @model
@@ -301,11 +321,10 @@ gnome_db_grid_new_with_gda_model (GnomeDbDict *dict, GdaDataModel *model)
 {
 	GtkWidget *grid;
 
-	if (dict)
-		g_return_val_if_fail (IS_GNOME_DB_DICT (dict), NULL);
+	g_return_val_if_fail (!dict || IS_GNOME_DB_DICT (dict), NULL);
 
 	grid = gnome_db_grid_new ();
-	gnome_db_grid_set_gda_model (GNOME_DB_GRID (grid), dict, model);
+	gnome_db_grid_widget_set_gda_model (GNOME_DB_DATA_WIDGET (grid), ASSERT_DICT (dict), model);
 
 	return grid;
 }
@@ -350,11 +369,17 @@ gnome_db_grid_new_with_select_query (GnomeDbQuery *query, GnomeDbTarget *modifie
 	g_object_unref (rs);
 	grid->priv->data_model = GNOME_DB_DATA_MODEL (rs);
 	grid->priv->proxy = proxy;
-
+	
 	grid->priv->data_set = gnome_db_data_model_get_new_data_set (GNOME_DB_DATA_MODEL (rs));
+	grid->priv->default_column_editable = !gnome_db_data_proxy_is_read_only (grid->priv->proxy);
 
 	gnome_db_grid_initialize (grid);
 
+#ifdef debug_NO
+	DEBUG_HEADER;
+	gnome_db_base_dump (rs, 5);
+#endif
+
 	return GTK_WIDGET (obj);
 }
 
@@ -379,9 +404,46 @@ gnome_db_grid_dispose (GObject *object)
 	parent_class->dispose (object);
 }
 
+static void
+gnome_db_grid_clean_row_refs (GnomeDbGrid *grid)
+{
+	if (grid->priv->nodes_gtk_row_refs) {
+		g_hash_table_destroy (grid->priv->nodes_gtk_row_refs);
+		grid->priv->nodes_gtk_row_refs = NULL;
+	}
+
+	if (grid->priv->row_ref_values) {
+		g_hash_table_destroy (grid->priv->row_ref_values);
+		grid->priv->row_ref_values = NULL;
+	}
+	
+	if (grid->priv->gtk_tree_row_references) {
+		GSList *list = grid->priv->gtk_tree_row_references;
+		while (list) {
+			gtk_tree_row_reference_free ((GtkTreeRowReference *)(list->data));
+			list = g_slist_next (list);
+		}
+		g_slist_free (grid->priv->gtk_tree_row_references);
+	}
+}
+
 static void
 gnome_db_grid_clean (GnomeDbGrid *grid)
 {
+	/* internally stored values */
+	gnome_db_grid_clean_row_refs (grid);
+
+	/* nodes_infos */
+	if (grid->priv->nodes_infos) {
+		GSList *list = grid->priv->nodes_infos;
+		while (list) {
+			utility_data_set_node_info_destroy (DATA_SET_NODE_INFO (list->data));
+			list = g_slist_next (list);
+		}
+		g_slist_free (grid->priv->nodes_infos);
+		grid->priv->nodes_infos = NULL;
+	}
+
 	/* info cells */
 	if (grid->priv->columns_data) {
 		GSList *list = grid->priv->columns_data;
@@ -445,15 +507,16 @@ gnome_db_grid_set_property (GObject              *object,
 				gtk_widget_hide (grid->priv->title);
                         break;
                 case PROP_TITLE_STRING:
-			gnome_db_gray_bar_set_text (GNOME_DB_GRAY_BAR (grid->priv->title), g_value_get_string (value));
-			gtk_widget_show (grid->priv->title);
+			gnome_db_grid_widget_set_title (GNOME_DB_DATA_WIDGET (grid), g_value_get_string (value));
                         break;
 		case PROP_ACTIONS_VISIBLE:
 			grid->priv->default_show_global_actions = g_value_get_boolean (value);
-			if (g_value_get_boolean (value)) 
-				gtk_widget_show (grid->priv->widget_group);
-			else
-				gtk_widget_hide (grid->priv->widget_group);
+			if (grid->priv->widget_group) {
+				if (g_value_get_boolean (value)) 
+					gtk_widget_show (grid->priv->widget_group);
+				else
+					gtk_widget_hide (grid->priv->widget_group);
+			}
 			break;
 		case PROP_INFO_CELL_VISIBLE: 
 		{
@@ -493,55 +556,6 @@ gnome_db_grid_get_property (GObject              *object,
         }	
 }
 
-
-/**
- * gnome_db_grid_set_title
- *
- * Returns: 
- */
-void
-gnome_db_grid_set_title (GnomeDbGrid *grid, const gchar *title)
-{
-	TO_IMPLEMENT;
-}
-
-/**
- * gnome_db_grid_set_gda_model
- */
-void
-gnome_db_grid_set_gda_model (GnomeDbGrid *grid, GnomeDbDict *dict, GdaDataModel *model)
-{
-	GnomeDbDataModel *gmodel;
-	GnomeDbDataProxy *proxy;
-
-	g_return_if_fail (grid && IS_GNOME_DB_GRID (grid));
-	g_return_if_fail (grid->priv);
-	if (dict)
-		g_return_if_fail (IS_GNOME_DB_DICT (dict));
-
-	gnome_db_grid_clean (grid);
-	gmodel = GNOME_DB_DATA_MODEL (gnome_db_model_wrapper_new (ASSERT_DICT (dict), model));
-	proxy = GNOME_DB_DATA_PROXY (gnome_db_data_proxy_new (gmodel));
-	g_object_unref (gmodel);
-	grid->priv->data_model = gmodel;
-	grid->priv->proxy = proxy;
-	grid->priv->data_set = gnome_db_data_model_get_new_data_set (gmodel);
-
-	gnome_db_grid_initialize (grid);
-}
-
-/**
- * gnome_db_grid_get_gda_model
- */
-GdaDataModel *
-gnome_db_grid_get_gda_model (GnomeDbGrid *grid)
-{
-	g_return_val_if_fail (grid && IS_GNOME_DB_GRID (grid), NULL);
-	g_return_val_if_fail (grid->priv, NULL);
-
-	return grid->priv->data_model;
-}
-
 /**
  * gnome_db_grid_get_selection
  * @grid: a #GnomeDbGrid widget
@@ -590,15 +604,6 @@ gnome_db_grid_get_selection (GnomeDbGrid *grid)
 		return NULL;
 }
 
-/**
- * gnome_db_grid_set_show_title
- */
-void              
-gnome_db_grid_set_show_title (GnomeDbGrid *grid, gboolean show)
-{
-	TO_IMPLEMENT;
-}
-
 /**
  * gnome_db_grid_foreach_selected
  */
@@ -627,25 +632,6 @@ gnome_db_grid_unselect_all (GnomeDbGrid *grid)
 	TO_IMPLEMENT;
 }
 
-/**
- * gnome_db_grid_set_show_title_icon
- */
-void
-gnome_db_grid_set_show_title_icon (GnomeDbGrid *grid, gboolean show)
-{
-	TO_IMPLEMENT;
-}
-
-/**
- * gnome_db_grid_set_title_icon_from_stock
- */
-void
-gnome_db_grid_set_title_icon_from_stock (GnomeDbGrid *grid, const gchar *stock_id, GtkIconSize size)
-{
-	TO_IMPLEMENT;
-}
-
-
 /* 
  * take care of param dependencies: look at all the parameters in the data set and 
  * if @param is a dependency of a parameter P, then P is set to NULL 
@@ -683,6 +669,18 @@ work_param_changed_cb (GnomeDbParameter *param, GnomeDbGrid *grid)
 	}
 }
 
+static void
+free_local_stored_values (GSList *values)
+{
+	GSList *list = values;
+	while (list) {
+		if (list->data)
+			gda_value_free ((GdaValue *)list->data);
+		list = g_slist_next (list);
+	}
+	g_slist_free (values);
+}
+
 /*
  * Real initialization
  */
@@ -690,10 +688,13 @@ static void
 gnome_db_grid_initialize (GnomeDbGrid *grid)
 {
 	GtkWidget *group, *sw;
+	GSList *list;
 
 	/* clean any widget, etc that were previously here */
-	if (grid->priv->title) 
-		gtk_widget_destroy (grid->priv->title);
+	if (grid->priv->title) {
+		gtk_widget_ref (grid->priv->title);
+		gtk_container_remove (GTK_CONTAINER (grid), grid->priv->title);
+	}
 	if (grid->priv->scroll) {
 		gtk_widget_destroy (grid->priv->scroll);
 		grid->priv->treeview = NULL;
@@ -706,11 +707,31 @@ gnome_db_grid_initialize (GnomeDbGrid *grid)
 		gtk_widget_destroy (grid->priv->widget_current_sample);
 	if (grid->priv->widget_group)
 		gtk_widget_destroy (grid->priv->widget_group);
+
+	/* DataSetNodeInfo info */
+	list = grid->priv->data_set->nodes;
+	while (list) {
+		if (GNOME_DB_DATA_SET_NODE (list->data)->params) {
+			DataSetNodeInfo *info;
+			info = utility_data_set_node_info_create (grid->priv->data_set, GNOME_DB_DATA_SET_NODE (list->data));
+			grid->priv->nodes_infos = g_slist_append (grid->priv->nodes_infos, info);
+		}
+		list = g_slist_next (list);
+	}
+
+	/* locally stored values */
+	grid->priv->nodes_gtk_row_refs = g_hash_table_new_full (NULL, NULL, NULL, (GDestroyNotify) g_slist_free);
+	grid->priv->row_ref_values = g_hash_table_new_full (NULL, NULL, NULL, (GDestroyNotify) free_local_stored_values);
 	
 	/* title */
-	grid->priv->title = gnome_db_gray_bar_new (_("No title"));
-        gtk_box_pack_start (GTK_BOX (grid), grid->priv->title, FALSE, TRUE, 2);
-	gtk_widget_show (grid->priv->title);
+	if (!grid->priv->title) {
+		grid->priv->title = gnome_db_gray_bar_new (_("No title"));
+		gtk_box_pack_start (GTK_BOX (grid), grid->priv->title, FALSE, TRUE, 2);
+	}
+	else {
+		gtk_box_pack_start (GTK_BOX (grid), grid->priv->title, FALSE, TRUE, 2);
+		gtk_widget_unref (grid->priv->title);
+	}
 
 	/*
 	 * TreeView preparation
@@ -733,11 +754,6 @@ gnome_db_grid_initialize (GnomeDbGrid *grid)
 		}
 	}
 
-	/* 
-	 * the grid's attributes
-	 */
-	gtk_widget_hide (grid->priv->title);
-
 	/*
 	 * the actions part
 	 */
@@ -798,6 +814,7 @@ create_new_tree_view (GnomeDbGrid *grid)
 	GtkTreeSelection *selection;
 	GSList *list;
 	GnomeDbDict *dict;
+	GSList *nodes_infos_list;
 
 	dict = gnome_db_base_get_dict (GNOME_DB_BASE (grid->priv->proxy));
 
@@ -822,6 +839,7 @@ create_new_tree_view (GnomeDbGrid *grid)
 			  G_CALLBACK (tree_model_row_changed_cb), grid);
 
 	/* Creation of the columns in the treeview, to fit the parameters in the context */
+	nodes_infos_list = grid->priv->nodes_infos;
 	list = grid->priv->data_set->nodes;
 	i = 0;
 	while (list) {
@@ -853,7 +871,11 @@ create_new_tree_view (GnomeDbGrid *grid)
 			type = gnome_db_parameter_get_data_type (param);
 			g_assert (type);
 
-			title = replace_double_underscores (g_strdup (gnome_db_base_get_name (GNOME_DB_BASE (param))));
+			title = gnome_db_base_get_name (GNOME_DB_BASE (param));
+			if (title && *title)
+				title = replace_double_underscores (g_strdup (title));
+			else
+				title = NULL;
 			if (!title)
 				title = g_strdup (_("No title"));
 			
@@ -913,8 +935,13 @@ create_new_tree_view (GnomeDbGrid *grid)
 
 			g_signal_connect (G_OBJECT (renderer), "changed", 
 					  G_CALLBACK (data_cell_values_changed), grid);
+
+			g_assert (nodes_infos_list);
+			g_object_set_data (G_OBJECT (column), "node_info", nodes_infos_list->data);
+			nodes_infos_list = g_slist_next (nodes_infos_list);
 		}
 		g_object_set_data (G_OBJECT (column), "data_renderer", renderer);
+		g_object_set (G_OBJECT (renderer), "editable", !column_data->data_locked, NULL);
 
 		/* settings and signals */
 		g_object_set (G_OBJECT (renderer), "editable", !column_data->data_locked, NULL);
@@ -924,7 +951,7 @@ create_new_tree_view (GnomeDbGrid *grid)
 		g_object_set_data (G_OBJECT (column), "context-node", node);
 
 		/* Adding the GdaValue's information cell as another GtkCellRenderer */
-		renderer = gnome_db_data_cell_renderer_info_new (grid->priv->proxy, i);
+		renderer = gnome_db_data_cell_renderer_info_new (grid->priv->proxy, grid->priv->data_set, node);
 		column_data->info_cell = renderer;
 		gtk_tree_view_column_pack_end (column, renderer, FALSE);
 		gtk_tree_view_column_set_cell_data_func (column, renderer, 
@@ -936,7 +963,7 @@ create_new_tree_view (GnomeDbGrid *grid)
 		g_object_set_data (G_OBJECT (renderer), "context-node", node);
 
 		/* Sorting data */
-		gtk_tree_view_column_set_sort_column_id (column, i);
+		/* gtk_tree_view_column_set_sort_column_id (column, i); */
 		/* gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (tree_model), */
 		/* i, (GtkTreeIterCompareFunc) tree_sortable_sort_values, node, NULL); */
 		list = g_slist_next (list);
@@ -966,6 +993,65 @@ create_new_tree_view (GnomeDbGrid *grid)
 	}
 }
 
+static GtkTreeRowReference *
+find_gtk_tree_row_reference (GnomeDbGrid *grid, GtkTreeIter *iter, GnomeDbDataSetNode *node, 
+			     gboolean create_if_missing, gboolean *created)
+{
+	GSList *list;
+	GtkTreeRowReference *ref = NULL;
+	GtkTreePath *refpath, *path;
+	
+	refpath = gtk_tree_model_get_path (GTK_TREE_MODEL (grid->priv->proxy), iter);
+	list = g_hash_table_lookup (grid->priv->nodes_gtk_row_refs, node);
+	while (list && !ref) {
+		path = gtk_tree_row_reference_get_path ((GtkTreeRowReference *) (list->data));
+		if (!path) {
+			/* remove this GtkTreeRowReference as it references an invalid path */
+			GtkTreeRowReference *rem_ref = (GtkTreeRowReference *) (list->data);
+
+			if (g_slist_length (list) <= 1) {
+				g_hash_table_remove (grid->priv->nodes_gtk_row_refs, node);
+				list = NULL;
+			}
+			else {
+				GSList *tmp = g_slist_copy (list);
+				tmp = g_slist_remove (tmp, rem_ref);
+				g_hash_table_insert (grid->priv->nodes_gtk_row_refs, node, tmp);
+				list = tmp;
+			}
+
+			g_hash_table_remove (grid->priv->row_ref_values, rem_ref);
+			grid->priv->gtk_tree_row_references = g_slist_remove (grid->priv->gtk_tree_row_references, rem_ref);
+			gtk_tree_row_reference_free (rem_ref);
+		}
+		else {
+			if (! gtk_tree_path_compare (path, refpath)) 
+				ref = (GtkTreeRowReference *) (list->data);
+			
+			gtk_tree_path_free (path);
+			list = g_slist_next (list);
+		}
+	}
+
+	if (created)
+		*created = FALSE;
+	
+	if (!ref && create_if_missing) {
+		list = g_hash_table_lookup (grid->priv->nodes_gtk_row_refs, node);
+		ref = gtk_tree_row_reference_new (GTK_TREE_MODEL (grid->priv->proxy), refpath);
+		list = g_slist_prepend (list, ref);
+		g_hash_table_insert (grid->priv->nodes_gtk_row_refs, node, list);
+		grid->priv->gtk_tree_row_references = g_slist_prepend (grid->priv->gtk_tree_row_references, ref);
+
+		if (created)
+			*created = TRUE;
+	}
+
+	gtk_tree_path_free (refpath);
+
+	return ref;
+}
+
 /*
  * Set the attributes for each cell renderer which is not the information cell renderer, 
  * called by each cell renderer before actually displaying anything.
@@ -995,9 +1081,9 @@ cell_renderer_value_set_attributes (GtkTreeViewColumn *tree_column,
 				    col, &value, 
 				    offset + col, &attributes, -1);
 		g_object_set (G_OBJECT (cell), 
+			      "value_attributes", attributes,
 			      "value", value,
 			      "editable", !column_data->data_locked,
-			      "value_attributes", attributes,
 			      "cell_background", GNOME_DB_COLOR_NORMAL_MODIF,
 			      "cell_background-set", ! (attributes & GNOME_DB_VALUE_IS_UNCHANGED),
 			      "to_be_deleted", to_be_deleted, 
@@ -1005,55 +1091,60 @@ cell_renderer_value_set_attributes (GtkTreeViewColumn *tree_column,
 	}
 	else { /* multiple parameters depending on a data model */
 		GList *values = NULL;
-		GSList *list;
+		DataSetNodeInfo *info;
+		gboolean values_to_free = TRUE;
 
-		if (IS_GNOME_DB_RESULT_SET (grid->priv->data_model)) {
-			/* list all the values of node->data_for_param which correspond to the values of
-			 * the parameters in node->params; this avoids a substantial later research */
-			gint i, max, col;
+		info = g_object_get_data (G_OBJECT (tree_column), "node_info");
 
-			max = gda_data_model_get_n_columns (GDA_DATA_MODEL (node->data_for_param));
-			for (i = 0 ; i < max; i++) {
-				col = gnome_db_result_set_find_column_ext (GNOME_DB_RESULT_SET (grid->priv->data_model),
-									   node, i);
-				g_assert (col >= 0);
-				gtk_tree_model_get (GTK_TREE_MODEL (grid->priv->proxy), iter, 
-						    PROXY_COL_TO_DELETE, &to_be_deleted,
-						    col, &value, 
-						    offset + col, &attributes, -1);
-				values = g_list_append (values, value);
+		/* NOTE:
+		 * For performances reasons we want to provide, if possible, all the values required by the combo cell
+		 * renderer to draw whatever it wants, without further making it search for the values it wants in
+		 * node->data_for_params.
+		 *
+		 * For this purpose, we try to get a complete list of values making one row of the node->data_for_params
+		 * data model, so that the combo cell renderer has all the values it wants.
+		 *
+		 * This optimization is possible if the utility_data_set_node_info_compute_values_ext() function
+		 * returns a non NULL list.
+		 * 
+		 * If it does then we try to look into the locally stored lists of values which we stored when
+		 * values of the combo cell renderer have changed (when data_cell_values_changed() was called), 
+		 * and we only store (for optimizations), such lists if the combo cell renderer made some changes.
+		 *
+		 * NOTE2:
+		 * This optimization is required anyway when node->data_for_params can be changed depending on
+		 * external events and we can't know when it has changed.
+		 */
+
+		attributes = utility_data_set_node_info_compute_attributes (info, grid->priv->proxy, iter, &to_be_deleted);
+		values = utility_data_set_node_info_compute_values_ext (info, grid->priv->proxy, iter);
+		if (!values) {
+			/* try if found in locally stored values */
+			GtkTreeRowReference *ref;
+			ref = find_gtk_tree_row_reference (grid, iter, node, FALSE, NULL);
+			if (ref) {
+				values = g_hash_table_lookup (grid->priv->row_ref_values, ref);
+				values_to_free = FALSE;
 			}
-
+		}
+		if (values) {
 			g_object_set (G_OBJECT (cell), 
+				      "value_attributes", attributes,
 				      "values_display", values,
 				      "editable", !column_data->data_locked,
-				      "value_attributes", attributes,
 				      "cell_background", GNOME_DB_COLOR_NORMAL_MODIF,
 				      "cell_background-set", ! (attributes & GNOME_DB_VALUE_IS_UNCHANGED),
 				      "to_be_deleted", to_be_deleted, 
 				      NULL);
-			g_list_free (values);
+			if (values_to_free)
+				g_list_free (values);
 		}
 		else {
-			/* list the values in grid->priv->data_model for each param in node->params */
-			list = node->params;
-			while (list) {
-				col = gnome_db_data_model_get_column_at_param (grid->priv->data_model, 
-									       grid->priv->data_set,
-									       list->data);
-				gtk_tree_model_get (GTK_TREE_MODEL (grid->priv->proxy), iter, 
-						    PROXY_COL_TO_DELETE, &to_be_deleted,
-						    col, &value, 
-						    offset + col, &attributes, -1);
-				values = g_list_append (values, value);
-				
-				list = g_slist_next (list);
-			}
-
+			values = utility_data_set_node_info_compute_values (info, grid->priv->proxy, iter);
 			g_object_set (G_OBJECT (cell), 
+				      "value_attributes", attributes,
 				      "values", values,
 				      "editable", !column_data->data_locked,
-				      "value_attributes", attributes,
 				      "cell_background", GNOME_DB_COLOR_NORMAL_MODIF,
 				      "cell_background-set", ! (attributes & GNOME_DB_VALUE_IS_UNCHANGED),
 				      "to_be_deleted", to_be_deleted, 
@@ -1098,13 +1189,11 @@ cell_renderer_info_set_attributes (GtkTreeViewColumn *tree_column,
 			      NULL);
 	}
 	else { /* multiple parameters depending on a data model */
-		/* REM: the attributes is arbitrary chosen to be the one from the first parameter */
-		col = gnome_db_data_model_get_column_at_param (grid->priv->data_model, grid->priv->data_set,
-							       GNOME_DB_PARAMETER (node->params->data));
-		gtk_tree_model_get (GTK_TREE_MODEL (grid->priv->proxy), iter, 
-				    PROXY_COL_TO_DELETE, &to_be_deleted,
-				    offset + col, &attributes, -1);
-		
+		DataSetNodeInfo *info;
+
+		info = g_object_get_data (G_OBJECT (tree_column), "node_info");
+		attributes = utility_data_set_node_info_compute_attributes (info, grid->priv->proxy, iter, &to_be_deleted);
+
 		g_object_set (G_OBJECT (cell), 
 			      "editable", !column_data->data_locked,
 			      "value_attributes", attributes,
@@ -1139,8 +1228,16 @@ data_cell_value_changed (GtkCellRenderer *renderer, const gchar *path, const Gda
 		    /* synchronize the parameters in data_set with the new values for the row */
 		    gnome_db_grid_set_params_from_iter (grid, &iter);
 	}
+
+#ifdef debug_NO
+	DEBUG_HEADER;
+	gnome_db_base_dump (grid->priv->proxy, 5);
+#endif
 }
 
+
+static void update_cached_gtk_tree_row_references (GnomeDbGrid *grid, GtkTreeIter *iter, GnomeDbDataSetNode *node, GSList *all_new_values);
+
 /*
  * Callback when a value displayed as a GnomeDbDataCellRendererCombo has been changed.
  *
@@ -1170,7 +1267,7 @@ data_cell_values_changed (GtkCellRenderer *renderer, const gchar *path,
 	if (set_iter_from_path (grid, path, &iter)) {
 		GSList *list, *params;
 		gint col;
-		
+
 		params = node->params;
 		list = new_values;
 		while (list) {
@@ -1181,22 +1278,11 @@ data_cell_values_changed (GtkCellRenderer *renderer, const gchar *path,
 			params = g_slist_next (params);
 		}
 		
-		if (IS_GNOME_DB_RESULT_SET (grid->priv->data_model)) {
-			/* also update the values of grid->priv->proxy which are copies of the values
-			 * which are displayed in the GnomeDbDataCellRendererCombo cell renderer, to keep
-			 * coherence. For this purpose, use @all_new_values */
-			gint i, max, col;
-			
-			list = all_new_values;
-			max = gda_data_model_get_n_columns (GDA_DATA_MODEL (node->data_for_param));
-			for (i = 0 ; i < max; i++) {
-				col = gnome_db_result_set_find_column_ext (GNOME_DB_RESULT_SET (grid->priv->data_model),
-									   node, i);
-				g_assert (col >= 0);
-				gnome_db_data_proxy_set_value (grid->priv->proxy, &iter, col, (GdaValue *)(list->data));
-				list = g_slist_next (list);
-			}
-		}
+		/* we need to internally store all these new values to be able to retreive them next time
+		 * we need them to redraw a particular cell renderer. See the explanation in 
+		 * cell_renderer_value_set_attributes()
+		 */
+		update_cached_gtk_tree_row_references (grid, &iter, node, all_new_values);
 
 		/* synchronize the parameters in data_set with the new values for the row */
 		gnome_db_grid_set_params_from_iter (grid, &iter);
@@ -1234,6 +1320,52 @@ set_iter_from_path (GnomeDbGrid *grid, const gchar *path, GtkTreeIter *iter)
 }
 
 
+static void
+update_cached_gtk_tree_row_references (GnomeDbGrid *grid, GtkTreeIter *iter, GnomeDbDataSetNode *node, GSList *all_new_values)
+{
+	GSList *list;
+	gboolean modified = FALSE;
+	GtkTreeRowReference *ref;
+	
+	if (all_new_values) {
+		gtk_tree_model_get (GTK_TREE_MODEL (grid->priv->proxy), iter, PROXY_COL_MODIFIED, &modified, -1);
+		if (modified) {
+			gboolean created;
+			GSList *tmp;
+			
+			ref = find_gtk_tree_row_reference (grid, iter, node, TRUE, &created);
+			list = NULL;
+			tmp = all_new_values;
+			while (tmp) {
+				if (tmp->data)
+					list = g_slist_append (list, gda_value_copy ((GdaValue *)(tmp->data)));
+				else
+					list = g_slist_append (list, NULL);
+				tmp = g_slist_next (tmp);
+			}
+			g_hash_table_insert (grid->priv->row_ref_values, ref, list);
+		}
+	}
+	if (!modified) {
+		/* row is now considered as non-modified */
+		ref = find_gtk_tree_row_reference (grid, iter, node, FALSE, NULL);
+		if (ref) {
+			list = g_hash_table_lookup (grid->priv->nodes_gtk_row_refs, node);
+			if (g_slist_length (list) <= 1)
+				g_hash_table_remove (grid->priv->nodes_gtk_row_refs, node);
+			else {
+				GSList *tmp = g_slist_copy (list);
+				tmp = g_slist_remove (tmp, ref);
+				g_hash_table_insert (grid->priv->nodes_gtk_row_refs, node, tmp);
+			}
+			
+			g_hash_table_remove (grid->priv->row_ref_values, ref);
+			grid->priv->gtk_tree_row_references = g_slist_remove (grid->priv->gtk_tree_row_references, ref);
+			gtk_tree_row_reference_free (ref);
+		}
+	}
+}
+
 /*
  * Apply the requested_action to the GtkTreeModel (grid->priv->proxy)
  */
@@ -1246,7 +1378,10 @@ data_cell_status_changed (GtkCellRenderer *renderer, const gchar *path, GnomeDbV
 	GnomeDbDataSetNode *node;
 	GtkTreeModel *tree_model;
 	gint col;
+	gint offset;
+	GdaValue *attribute;
 
+        offset = gnome_db_data_proxy_get_n_columns (grid->priv->proxy);
 	node = g_object_get_data (G_OBJECT (renderer), "context-node");
 	tree_model = GTK_TREE_MODEL (grid->priv->proxy);
 
@@ -1258,16 +1393,32 @@ data_cell_status_changed (GtkCellRenderer *renderer, const gchar *path, GnomeDbV
 	}
 	gtk_tree_path_free (treepath);
 	
-
+	attribute = gda_value_new_uinteger (requested_action);
 	if (node->param) {
 		col = gnome_db_data_model_get_column_at_param (grid->priv->data_model, grid->priv->data_set, node->param);
-		TO_IMPLEMENT;
+		gnome_db_data_proxy_set_value (grid->priv->proxy, &iter, offset + col, attribute);
 	}
-	else
-		TO_IMPLEMENT;
+	else {
+		GSList *list = node->params;
+		while (list) {
+			col = gnome_db_data_model_get_column_at_param (grid->priv->data_model, grid->priv->data_set, 
+								       GNOME_DB_PARAMETER (list->data));
+			gnome_db_data_proxy_set_value (grid->priv->proxy, &iter, offset + col, attribute);
+			list = g_slist_next (list);
+		}
+		
+		/* we need to internally remove any cached list of values */
+		update_cached_gtk_tree_row_references (grid, &iter, node, NULL);
+	}
+	gda_value_free (attribute);
 
 	/* synchronize the parameters in data_set with the new values for the row */
 	gnome_db_grid_set_params_from_iter (grid, &iter);
+
+#ifdef debug_NO
+	DEBUG_HEADER;
+	gnome_db_base_dump (grid->priv->proxy, 0);
+#endif
 }
 
 /* static gint */
@@ -1454,8 +1605,16 @@ tree_view_selection_changed_cb (GtkTreeSelection *selection, GnomeDbGrid *grid)
 static void
 tree_model_row_changed_cb (GtkTreeModel *treemodel, GtkTreePath *arg1, GtkTreeIter *arg2, GnomeDbGrid *grid)
 {
-	/* update the actions buttons */
-	modif_buttons_update (grid);
+	GtkTreeSelection *selection;
+
+	/* if the changed row is the current selection, then update the data set */
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (grid->priv->treeview));
+	if (gtk_tree_selection_iter_is_selected (selection, arg2) &&
+	    (gtk_tree_selection_count_selected_rows (selection) == 1))
+		tree_view_selection_changed_cb (selection, grid);
+	else
+		/* only update the actions buttons */
+		modif_buttons_update (grid);	    
 }
 
 /*
@@ -1869,6 +2028,7 @@ gnome_db_grid_set_column_editable (GnomeDbDataWidget *iface, gint column, gboole
 	g_return_if_fail (grid->priv);
 	
 	if (grid->priv->data_model) {
+		editable = editable && !gnome_db_data_proxy_is_read_only (grid->priv->proxy);
 		param = gnome_db_data_model_get_param_at_column (grid->priv->data_model, grid->priv->data_set, column);
 		node = gnome_db_data_set_find_node_for_param (grid->priv->data_set, param);
 		g_return_if_fail (node);
@@ -1882,8 +2042,8 @@ gnome_db_grid_set_column_editable (GnomeDbDataWidget *iface, gint column, gboole
 		
 		g_object_set (G_OBJECT (column_data->data_cell), "editable", !column_data->data_locked, NULL);
 	}
-	else
-		grid->priv->default_column_editable = editable;
+
+	grid->priv->default_column_editable = editable;
 }
 
 static void
@@ -1999,3 +2159,89 @@ gnome_db_grid_widget_get_data_set (GnomeDbDataWidget *iface)
 
 	return grid->priv->data_set;
 }
+
+
+static GdaDataModel *
+gnome_db_grid_widget_get_gda_model (GnomeDbDataWidget *iface)
+{
+	GnomeDbGrid *grid;
+	g_return_val_if_fail (iface && IS_GNOME_DB_GRID (iface), NULL);
+	grid = GNOME_DB_GRID (iface);
+	g_return_val_if_fail (grid->priv, NULL);
+
+	/* warning: here we are not returning the *original* GdaDataModel, but one one encapsulated
+	 * by the GnomeDbDataModel; maybe we'll have to change this */
+
+	return GDA_DATA_MODEL (grid->priv->data_model);
+}
+
+static void
+gnome_db_grid_widget_set_gda_model (GnomeDbDataWidget *iface, GnomeDbDict *dict, GdaDataModel *model)
+{
+	GnomeDbGrid *grid;
+	GnomeDbDataModel *gmodel;
+	GnomeDbDataProxy *proxy;
+
+	g_return_if_fail (iface && IS_GNOME_DB_GRID (iface));
+	grid = GNOME_DB_GRID (iface);
+	g_return_if_fail (grid->priv);
+	g_return_if_fail (!dict || IS_GNOME_DB_DICT (dict));
+	
+	gnome_db_grid_clean (grid);
+	gmodel = GNOME_DB_DATA_MODEL (gnome_db_model_wrapper_new (ASSERT_DICT (dict), model));
+	proxy = GNOME_DB_DATA_PROXY (gnome_db_data_proxy_new (gmodel));
+	g_object_unref (gmodel);
+	grid->priv->data_model = gmodel;
+	grid->priv->proxy = proxy;
+	grid->priv->data_set = gnome_db_data_model_get_new_data_set (gmodel);
+
+	if (grid->priv->default_column_editable)
+		grid->priv->default_column_editable = !gnome_db_data_proxy_is_read_only (grid->priv->proxy);
+	gnome_db_grid_initialize (grid);
+}
+
+static void
+gnome_db_grid_widget_set_title (GnomeDbDataWidget *iface, const gchar *title)
+{
+	GnomeDbGrid *grid;
+	g_return_if_fail (iface && IS_GNOME_DB_GRID (iface));
+	grid = GNOME_DB_GRID (iface);
+	g_return_if_fail (grid->priv);
+
+	gnome_db_gray_bar_set_text (GNOME_DB_GRAY_BAR (grid->priv->title), title);
+	if (title && *title)
+		gtk_widget_show (grid->priv->title);
+	else
+		gtk_widget_hide (grid->priv->title);	
+}
+
+static void
+gnome_db_grid_widget_set_show_title (GnomeDbDataWidget *iface, gboolean show)
+{
+	GnomeDbGrid *grid;
+	g_return_if_fail (iface && IS_GNOME_DB_GRID (iface));
+	grid = GNOME_DB_GRID (iface);
+	g_return_if_fail (grid->priv);
+
+	if (show)
+		gtk_widget_show (grid->priv->title);
+	else
+		gtk_widget_hide (grid->priv->title);
+}
+
+static void
+gnome_db_grid_widget_set_title_icon_from_stock (GnomeDbDataWidget *iface, const gchar *stock_id, GtkIconSize size)
+{
+	GnomeDbGrid *grid;
+	g_return_if_fail (iface && IS_GNOME_DB_GRID (iface));
+	grid = GNOME_DB_GRID (iface);
+	g_return_if_fail (grid->priv);
+	
+	if (stock_id && *stock_id) {
+		gnome_db_gray_bar_set_icon_from_stock (GNOME_DB_GRAY_BAR (grid->priv->title),
+						       stock_id, size);
+		gnome_db_gray_bar_set_show_icon (GNOME_DB_GRAY_BAR (grid->priv->title), TRUE);
+	}
+	else
+		gnome_db_gray_bar_set_show_icon (GNOME_DB_GRAY_BAR (grid->priv->title), FALSE);
+}
