@@ -31,7 +31,6 @@
 #include <libgnomeui/gnome-ui-init.h>
 #endif
 #include <libgnomedb/libgnomedb.h>
-#include "gnome-db-i18n.h"
 
 extern void gnome_db_stock_init (void);
 GnomeDbDict *default_dict = NULL; /* available in all libgnomedb, anways NOT NULL */
@@ -43,9 +42,9 @@ GnomeDbDict *default_dict = NULL; /* available in all libgnomedb, anways NOT NUL
 /**
  * gnome_db_get_default_dict
  *
- * Get the default dictionnary.
+ * Get the default dictionary.
  *
- * Returns: a not %NULL pointer to the default #GnomeDbDict dictionnary
+ * Returns: a not %NULL pointer to the default #GnomeDbDict dictionary
  */
 GnomeDbDict *
 gnome_db_get_default_dict (void)
