@@ -234,10 +234,10 @@ gnome_db_parameter_new (GnomeDbDict *dict, GnomeDbServerDataType *type)
 	GObject   *obj;
 	GnomeDbParameter *param;
 
-        g_return_val_if_fail (dict && IS_GNOME_DB_DICT (dict), NULL);
+        g_return_val_if_fail (!dict || IS_GNOME_DB_DICT (dict), NULL);
 	g_return_val_if_fail (type && IS_GNOME_DB_SERVER_DATA_TYPE (type), NULL);
 
-        obj = g_object_new (GNOME_DB_PARAMETER_TYPE, "dict", dict, NULL);
+        obj = g_object_new (GNOME_DB_PARAMETER_TYPE, "dict", ASSERT_DICT (dict), NULL);
 	param = GNOME_DB_PARAMETER (obj);
 
 	gnome_db_parameter_set_data_type (param, type);
@@ -477,6 +477,7 @@ gnome_db_parameter_set_property    (GObject              *object,
 			break;
 		case PROP_USE_DEFAULT_VALUE:
 			parameter->priv->default_forced = g_value_get_boolean (value);
+			parameter->priv->invalid_forced = FALSE;
 			break;
 		case PROP_SIMPLE_BIND:
 			gnome_db_parameter_bind_to_param (parameter, g_value_get_pointer (value));
@@ -644,7 +645,7 @@ gnome_db_parameter_set_value (GnomeDbParameter *param, const GdaValue *value)
 		 current_val ? gda_value_stringify (current_val) : "_NULL_",
 		 value ? gda_value_stringify (value) : "_NULL_",
 		 current_val ? gda_value_get_type (current_val) : 0,
-		 value ? gda_value_get_type (value) : 0, param->priv->valid, changed);
+		 value ? gda_value_get_type (value) : 0, gnome_db_parameter_is_valid (param), changed);
 #endif
 
 	/* end of procedure if the value has not been changed, after calculating the param's validity */
@@ -743,16 +744,20 @@ gnome_db_parameter_is_valid (GnomeDbParameter *param)
 	g_return_val_if_fail (param && IS_GNOME_DB_PARAMETER (param), FALSE);
 	g_return_val_if_fail (param->priv, FALSE);
 
-	if (param->priv->alias_of)
+	if (param->priv->alias_of) {
 		return gnome_db_parameter_is_valid (param->priv->alias_of);
+	}
 	else {
-		if (param->priv->invalid_forced)
+		if (param->priv->invalid_forced) {
 			return FALSE;
+		}
 
-		if (param->priv->default_forced) 
+		if (param->priv->default_forced) {
 			return param->priv->default_value ? TRUE : FALSE;
-		else
+		}
+		else {
 			return param->priv->valid;
+		}
 	}
 }
 
@@ -1076,6 +1081,7 @@ gnome_db_parameter_bind_to_param (GnomeDbParameter *param, GnomeDbParameter *bin
 
 	g_return_if_fail (param && IS_GNOME_DB_PARAMETER (param));
 	g_return_if_fail (param->priv);
+	g_return_if_fail (param != bind_to);
 
 	if (param->priv->change_with == bind_to)
 		return;
@@ -1328,13 +1334,21 @@ gnome_db_parameter_dump (GnomeDbParameter *parameter, guint offset)
 		
 		list = parameter->priv->for_fields;
 		while (list) {
-			gchar *xmlid = gnome_db_xml_storage_get_xml_id (GNOME_DB_XML_STORAGE (list->data));
+			gchar *xmlid = NULL;
+
+			if (IS_GNOME_DB_XML_STORAGE (list->data))
+				xmlid = gnome_db_xml_storage_get_xml_id (GNOME_DB_XML_STORAGE (list->data));
+
 			if (list == parameter->priv->for_fields)
 				g_print ("%sFor fields: ", str);
 			else
 				g_print (", ");
-			g_print ("%p (%s)", list->data, xmlid);
-			g_free (xmlid);
+			if (xmlid) {
+				g_print ("%p (%s)", list->data, xmlid);
+				g_free (xmlid);
+			}
+			else
+				g_print ("%p", list->data);
 
 			list = g_slist_next (list);
 		}
@@ -1419,6 +1433,7 @@ gnome_db_parameter_replace_refs (GnomeDbReferer *iface, GHashTable *replacements
 
 	list = parameter->priv->for_fields;
 	while (list) {
+		/* FIXME: the parameter->priv->for_fields is changed during the loop! */
 		repl = g_hash_table_lookup (replacements, list->data);
 		if (repl) {
 			gnome_db_parameter_del_for_field (parameter, GNOME_DB_FIELD (list->data));
