@@ -84,8 +84,8 @@ static gboolean    gnome_db_qf_value_is_equal       (GnomeDbQfield *qfield1, Gno
 static void nullified_object_cb (GObject *obj, GnomeDbQfValue *field);
 
 static GSList   *gnome_db_qf_value_get_params (GnomeDbQfield *qfield);
-static gboolean  gnome_db_qf_value_set_force_param (GnomeDbQfValue *field, GnomeDbQfield *provider, GError **error);
-static gboolean  gnome_db_qf_value_set_force_param_xml (GnomeDbQfValue *field, const gchar *prov_xml_id, GError **error);
+static gboolean  gnome_db_qf_value_restrict_with_field (GnomeDbQfValue *field, GnomeDbQfield *provider, GError **error);
+static gboolean  gnome_db_qf_value_restrict_with_field_xml (GnomeDbQfValue *field, const gchar *prov_xml_id, GError **error);
 
 static gboolean gnome_db_qf_value_render_find_value (GnomeDbQfValue *field, GnomeDbDataSet *context,
 					       const GdaValue **value_found, GnomeDbParameter **param_source);
@@ -366,7 +366,7 @@ gnome_db_qf_value_dispose (GObject *object)
 			gnome_db_qf_value->priv->default_value = NULL;
 		}
 		if (gnome_db_qf_value->priv->value_prov_ref)
-			gnome_db_qf_value_set_force_param (gnome_db_qf_value, NULL, NULL);
+			gnome_db_qf_value_restrict_with_field (gnome_db_qf_value, NULL, NULL);
 	}
 
 	/* parent class */
@@ -426,11 +426,11 @@ gnome_db_qf_value_set_property (GObject              *object,
 			break;
 		case PROP_VALUE_PROVIDER_OBJECT:
 			ptr = g_value_get_pointer (value);
-			g_return_if_fail (gnome_db_qf_value_set_force_param (gnome_db_qf_value, (GnomeDbQfield *) ptr, NULL));
+			g_return_if_fail (gnome_db_qf_value_restrict_with_field (gnome_db_qf_value, (GnomeDbQfield *) ptr, NULL));
 			break;
 		case PROP_VALUE_PROVIDER_XML_ID:
 			str =  g_value_get_string (value);
-			g_return_if_fail (gnome_db_qf_value_set_force_param_xml (gnome_db_qf_value, str, NULL));
+			g_return_if_fail (gnome_db_qf_value_restrict_with_field_xml (gnome_db_qf_value, str, NULL));
 			break;
 		case PROP_HANDLER_PLUGIN:
 			str =  g_value_get_string (value);
@@ -500,10 +500,10 @@ gnome_db_qf_value_copy (GnomeDbQfield *orig)
 		GnomeDbBase *ref = gnome_db_ref_base_get_ref_object (qf->priv->value_prov_ref);
 		if (ref) {
 			g_assert (IS_GNOME_DB_QFIELD (ref));
-			gnome_db_qf_value_set_force_param (nqf, GNOME_DB_QFIELD (ref), NULL);
+			gnome_db_qf_value_restrict_with_field (nqf, GNOME_DB_QFIELD (ref), NULL);
 		}
 		else
-			gnome_db_qf_value_set_force_param_xml (nqf, 
+			gnome_db_qf_value_restrict_with_field_xml (nqf, 
 							 gnome_db_ref_base_get_ref_name (qf->priv->value_prov_ref, NULL, NULL),
 							 NULL);
 	}
@@ -815,19 +815,19 @@ gnome_db_qf_value_is_value_null (GnomeDbQfValue *field, GnomeDbDataSet *context)
 }
 
 /*
- * gnome_db_qf_value_set_force_param
+ * gnome_db_qf_value_restrict_with_field
  * @field: a #GnomeDbQfValue object
  * @provider: a #GnomeDbQfield object, or NULL
  * @error: location to store error, or %NULL
  *
- * Forces the @field parameter's value to be in the possible values represented by @provider.
+ * Restrict the @field's value to be in the possible values represented by @provider.
  * @provider must be a field in another entity (a #GnomeDbQuery listed as a parameter source for @field's
- * own query). This other entity must be a selection query.
+ * own query).
  *
  * Returns: TRUE if no error occurred
  */
 static gboolean
-gnome_db_qf_value_set_force_param (GnomeDbQfValue *field, GnomeDbQfield *provider, GError **error)
+gnome_db_qf_value_restrict_with_field (GnomeDbQfValue *field, GnomeDbQfield *provider, GError **error)
 {
 	g_return_val_if_fail (field && IS_GNOME_DB_QF_VALUE (field), FALSE);
 	g_return_val_if_fail (field->priv, FALSE);
@@ -843,6 +843,7 @@ gnome_db_qf_value_set_force_param (GnomeDbQfValue *field, GnomeDbQfield *provide
 		GnomeDbServerDataType *dt;
 
 		/* Specific tests */
+		/* FIXME */
 		g_return_val_if_fail (IS_GNOME_DB_QFIELD (provider), FALSE);
 		
 		if (!gnome_db_qfield_is_visible (provider)) {
@@ -856,7 +857,7 @@ gnome_db_qf_value_set_force_param (GnomeDbQfValue *field, GnomeDbQfield *provide
 		entity = gnome_db_field_get_entity (GNOME_DB_FIELD (provider));
 		g_return_val_if_fail (IS_GNOME_DB_QUERY (entity), FALSE);
 		query = GNOME_DB_QUERY (entity);
-
+		
 		if (!gnome_db_query_is_select_query (query)) {
 			g_set_error (error,
 				     GNOME_DB_QF_VALUE_ERROR,
@@ -867,13 +868,14 @@ gnome_db_qf_value_set_force_param (GnomeDbQfValue *field, GnomeDbQfield *provide
 		
 		dt = gnome_db_field_get_data_type (GNOME_DB_FIELD (provider));
 		if (dt && (dt != field->priv->srv_type)) {
-				g_set_error (error,
+			g_set_error (error,
 				     GNOME_DB_QF_VALUE_ERROR,
 				     GNOME_DB_QF_VALUE_PARAM_ERROR,
 				     _("Incompatible field type for a parameter's provider (%s / %s)"), 
-				     gnome_db_server_data_type_get_sqlname (dt), gnome_db_server_data_type_get_sqlname (field->priv->srv_type));
-				return FALSE;
-			}
+				     gnome_db_server_data_type_get_sqlname (dt),
+				     gnome_db_server_data_type_get_sqlname (field->priv->srv_type));
+			return FALSE;
+		}
 		
 		field->priv->value_prov_ref = GNOME_DB_REF_BASE (gnome_db_ref_base_new (gnome_db_base_get_dict (GNOME_DB_BASE (field))));
 		gnome_db_ref_base_set_ref_object_type (field->priv->value_prov_ref, GNOME_DB_BASE (provider), GNOME_DB_FIELD_TYPE);
@@ -883,17 +885,17 @@ gnome_db_qf_value_set_force_param (GnomeDbQfValue *field, GnomeDbQfield *provide
 }
 
 /*
- * gnome_db_qf_value_set_force_param_xml
+ * gnome_db_qf_value_restrict_with_field_xml
  * @field: a #GnomeDbQfValue object
  * @prov_xml_id: the XML Id of a #GnomeDbDfield object, or NULL
  * @error: location to store error, or %NULL
  *
- * Same as gnome_db_qf_value_set_force_param() but using an XML Id instead of a direct pointer
+ * Same as gnome_db_qf_value_restrict_with_field() but using an XML Id instead of a direct pointer
  *
  * Returns: TRUE if no error occurred
  */
 static gboolean 
-gnome_db_qf_value_set_force_param_xml (GnomeDbQfValue *field, const gchar *prov_xml_id, GError **error)
+gnome_db_qf_value_restrict_with_field_xml (GnomeDbQfValue *field, const gchar *prov_xml_id, GError **error)
 {
 	g_return_val_if_fail (field && IS_GNOME_DB_QF_VALUE (field), FALSE);
 	g_return_val_if_fail (field->priv, FALSE);
@@ -907,7 +909,7 @@ gnome_db_qf_value_set_force_param_xml (GnomeDbQfValue *field, const gchar *prov_
 		/* No test possible, really */
 		field->priv->value_prov_ref = GNOME_DB_REF_BASE (gnome_db_ref_base_new (gnome_db_base_get_dict (GNOME_DB_BASE (field))));
 		gnome_db_ref_base_set_ref_name (field->priv->value_prov_ref, GNOME_DB_FIELD_TYPE, REFERENCE_BY_XML_ID,
-					  prov_xml_id);
+						prov_xml_id);
 	}
 
 	return TRUE;	
@@ -1256,7 +1258,7 @@ gnome_db_qf_value_load_from_xml (GnomeDbXmlStorage *iface, xmlNodePtr node, GErr
 
 	prop = xmlGetProp (node, "value_prov");
 	if (prop) {
-		if (!gnome_db_qf_value_set_force_param_xml (field, prop, error))
+		if (!gnome_db_qf_value_restrict_with_field_xml (field, prop, error))
 			err = TRUE;
 		g_free (prop);
 	}
