@@ -638,6 +638,10 @@ gnome_db_qfield_is_equal (GnomeDbQfield *qfield1, GnomeDbQfield *qfield2)
 	g_return_val_if_fail (qfield2 && IS_GNOME_DB_QFIELD (qfield2), FALSE);
 	g_return_val_if_fail (qfield1->priv, FALSE);
 	g_return_val_if_fail (qfield2->priv, FALSE);
+	
+	if (qfield1 == qfield2)
+		return TRUE;
+
 	g_object_get (G_OBJECT (qfield1), "query", &q1, NULL);
 	g_object_get (G_OBJECT (qfield2), "query", &q2, NULL);
 	g_return_val_if_fail (q1, FALSE);
