@@ -1,6 +1,6 @@
 /* gnome-db-selector.c
  *
- * Copyright (C) 2002 - 2004 Vivien Malerba
+ * Copyright (C) 2002 - 2005 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -149,12 +149,12 @@ gnome_db_selector_new (GnomeDbDict *dict, GObject *ref_object, gulong mode, gulo
 	GObject    *obj;
 	GnomeDbSelector *mgsel;
 
-	g_return_val_if_fail (dict && IS_GNOME_DB_DICT (dict), NULL);
+	g_return_val_if_fail (!dict || IS_GNOME_DB_DICT (dict), NULL);
 		
 	obj = g_object_new (GNOME_DB_SELECTOR_TYPE, NULL);
 	mgsel = GNOME_DB_SELECTOR (obj);
 
-	mgsel->priv->dict = dict;
+	mgsel->priv->dict = ASSERT_DICT (dict);
 	mgsel->priv->mode = mode;
 	mgsel->priv->columns = columns;
 
