@@ -285,8 +285,8 @@ gnome_db_server_class_init (GnomeDbServerClass * class)
                               G_TYPE_NONE, 1, G_TYPE_POINTER);
         gnome_db_server_signals[DATA_AGGREGATE_REMOVED] =
                 g_signal_new ("data_aggregate_removed",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_FIRST,
+                                G_TYPE_FROM_CLASS (object_class),
+                            G_SIGNAL_RUN_FIRST,
                               G_STRUCT_OFFSET (GnomeDbServerClass, data_aggregate_removed),
                               NULL, NULL,
                               marshal_VOID__POINTER,
@@ -528,12 +528,12 @@ gnome_db_server_new (GnomeDbDict *dict)
 	GObject   *obj;
 	GnomeDbServer *gnome_db_server;
 
-	g_return_val_if_fail (dict && IS_GNOME_DB_DICT (dict), NULL);
+	g_return_val_if_fail (!dict || IS_GNOME_DB_DICT (dict), NULL);
 
 	obj = g_object_new (GNOME_DB_SERVER_TYPE, NULL);
 	gnome_db_server = GNOME_DB_SERVER (obj);
 
-	gnome_db_server->priv->dict = dict;
+	gnome_db_server->priv->dict = ASSERT_DICT (dict);
 
 	return obj;
 }
@@ -998,7 +998,7 @@ gnome_db_server_get_dict (GnomeDbServer *srv)
  * If the requested datasource does not exist, then nothing is done ans FALSE
  * is returned.
  *
- * If the default XML filename to save the dictionnary has not yet been
+ * If the default XML filename to save the dictionary has not yet been
  * specified, then the default one is specified, so it is possible to call
  * gnome_db_dict_load_xml() right after that.
  *
@@ -1029,7 +1029,7 @@ gnome_db_server_set_datasource (GnomeDbServer *srv, const gchar *datasource)
 	srv->priv->info = gnome_db_server_info_get (dsn->provider);
 	gda_config_free_data_source_info (dsn);
 
-	/* set the default XML filename for the dictionnary */
+	/* set the default XML filename for the dictionary */
 	cstr = gnome_db_dict_get_xml_filename (srv->priv->dict);
 	if (!cstr) {
 		gchar *str;
