@@ -246,11 +246,11 @@ gnome_db_table_field_new (GnomeDbDict *dict, GnomeDbServerDataType *type)
 	GObject   *obj;
 	GnomeDbTableField *gnome_db_table_field;
 
-	g_return_val_if_fail (dict && IS_GNOME_DB_DICT (dict), NULL);
+	g_return_val_if_fail (!dict || IS_GNOME_DB_DICT (dict), NULL);
 	if (type)
 		g_return_val_if_fail (IS_GNOME_DB_SERVER_DATA_TYPE (type), NULL);
 
-	obj = g_object_new (GNOME_DB_TABLE_FIELD_TYPE, "dict", dict, NULL);
+	obj = g_object_new (GNOME_DB_TABLE_FIELD_TYPE, "dict", ASSERT_DICT (dict), NULL);
 	gnome_db_table_field = GNOME_DB_TABLE_FIELD (obj);
 	gnome_db_base_set_id (GNOME_DB_BASE (gnome_db_table_field), 0);
 
@@ -258,7 +258,8 @@ gnome_db_table_field_new (GnomeDbDict *dict, GnomeDbServerDataType *type)
 		gnome_db_table_field_set_data_type (gnome_db_table_field, type);
 
 	/* Extra init */
-	gnome_db_server_set_object_func_handler (gnome_db_dict_get_server (dict), gnome_db_table_field_handler_func);
+	gnome_db_server_set_object_func_handler (gnome_db_dict_get_server (ASSERT_DICT (dict)), 
+						 gnome_db_table_field_handler_func);
 	
 	return obj;
 }
