@@ -251,9 +251,9 @@ gnome_db_table_new (GnomeDbDict *dict)
 	GObject   *obj;
 	GnomeDbTable *gnome_db_table;
 
-	g_return_val_if_fail (dict && IS_GNOME_DB_DICT (dict), NULL);
+	g_return_val_if_fail (!dict || IS_GNOME_DB_DICT (dict), NULL);
 
-	obj = g_object_new (GNOME_DB_TABLE_TYPE, "dict", dict, NULL);
+	obj = g_object_new (GNOME_DB_TABLE_TYPE, "dict", ASSERT_DICT (dict), NULL);
 	gnome_db_table = GNOME_DB_TABLE (obj);
 	gnome_db_base_set_id (GNOME_DB_BASE (gnome_db_table), 0);
 
