@@ -52,12 +52,13 @@
 #include <gtk/gtktextbuffer.h>
 #include <gtk/gtktreeview.h>
 #include <gtk/gtkvbox.h>
+#include <gtk/gtkversion.h>
+#if GTK_CHECK_VERSION(2, 4, 0)
+#  include <gtk/gtkfilechooserdialog.h>
+#endif
 #include <gtk/gtkvpaned.h>
 #include <gtk/gtkvseparator.h>
 #include <gtk/gtkwindow.h>
-#ifdef BUILD_WITH_GNOME
-#include <libgnomedb/gnome-db-browser.h>
-#endif
 #include <libgnomedb/gnome-db-grid.h>
 #include <libgnomedb/gnome-db-util.h>
 #ifdef BUILD_WITH_GNOME
@@ -65,24 +66,13 @@
 #endif
 #include "gnome-db-i18n.h"
 
-/**
- * gnome_db_new_browser_widget
- */
-#ifdef BUILD_WITH_GNOME
-GtkWidget *
-gnome_db_new_browser_widget (void)
-{
-	GtkWidget *browser;
-
-	browser = gnome_db_browser_new ();
-	gtk_widget_show (browser);
-
-	return browser;
-}
-#endif
-
 /**
  * gnome_db_new_button_widget
+ * @label:
+ *
+ *
+ *
+ * Returns:
  */
 GtkWidget *
 gnome_db_new_button_widget (const gchar *label)
@@ -100,6 +90,11 @@ gnome_db_new_button_widget (const gchar *label)
 
 /**
  * gnome_db_new_button_widget_from_stock
+ * @stock_id:
+ *
+ *
+ *
+ * Returns:
  */
 GtkWidget *
 gnome_db_new_button_widget_from_stock (const gchar *stock_id)
@@ -112,39 +107,15 @@ gnome_db_new_button_widget_from_stock (const gchar *stock_id)
 	return button;
 }
 
-/**
- * gnome_db_new_check_button_widget
- */
-GtkWidget *
-gnome_db_new_check_button_widget (const gchar *label)
-{
-	GtkWidget *check;
-
-	if (label != NULL)
-		check = gtk_check_button_new_with_label (label);
-	else
-		check = gtk_check_button_new ();
-	gtk_widget_show (check);
-
-	return check;
-}
-
-/**
- * gnome_db_new_combo_widget
- */
-GtkWidget *
-gnome_db_new_combo_widget (void)
-{
-	GtkWidget *combo;
-
-	combo = gtk_combo_new ();
-	gtk_widget_show (combo);
-
-	return combo;
-}
 
 /**
  * gnome_db_new_entry_widget
+ * @max_length:
+ * @editable:
+ *
+ *
+ *
+ * Returns:
  */
 GtkWidget *
 gnome_db_new_entry_widget (gint max_length, gboolean editable)
@@ -163,52 +134,43 @@ gnome_db_new_entry_widget (gint max_length, gboolean editable)
 #ifdef BUILD_WITH_GNOME
 /**
  * gnome_db_new_file_entry_widget
+ * @history_id:
+ * @title:
+ *
+ *
+ *
+ * Returns:
  */
 GtkWidget *
 gnome_db_new_file_entry_widget (const gchar *history_id, const gchar *title)
 {
 	GtkWidget *entry;
+#if GTK_CHECK_VERSION(2,4,0)
+	GValue value = { 0, };
+#endif
 
 	entry = gnome_file_entry_new (history_id, title);
+#if GTK_CHECK_VERSION(2, 4, 0)
+	g_value_init (&value, G_TYPE_BOOLEAN);
+	g_value_set_boolean (&value, TRUE);
+	g_object_set_property (G_OBJECT (entry), "use-filechooser", &value);
+	g_value_reset (&value);
+#endif
 	gtk_widget_show (entry);
 
 	return entry;
 }
 #endif
 
-/**
- * gnome_db_new_frame_widget
- */
-GtkWidget *
-gnome_db_new_frame_widget (const gchar *title)
-{
-	GtkWidget *frame;
-
-	frame = gtk_frame_new (title);
-	gtk_widget_show (frame);
-
-	return frame;
-}
-
-/**
- * gnome_db_new_grid_widget
- */
-GtkWidget *
-gnome_db_new_grid_widget (GdaDataModel *model)
-{
-	GtkWidget *grid;
-
-	if (GDA_IS_DATA_MODEL (model))
-		grid = gnome_db_grid_new_with_model (model);
-	else
-		grid = gnome_db_grid_new ();
-	gtk_widget_show (grid);
-
-	return grid;
-}
 
 /**
  * gnome_db_new_hbox_widget
+ * @homogenous:
+ * @spacing:
+ *
+ *
+ *
+ * Returns:
  */
 GtkWidget *
 gnome_db_new_hbox_widget (gboolean homogenous, gint spacing)
@@ -221,50 +183,13 @@ gnome_db_new_hbox_widget (gboolean homogenous, gint spacing)
 	return hbox;
 }
 
-/**
- * gnome_db_new_hbutton_box_widget
- */
-GtkWidget *
-gnome_db_new_hbutton_box_widget (void)
-{
-	GtkWidget *hbbox;
-
-	hbbox = gtk_hbutton_box_new ();
-	gtk_widget_show (hbbox);
-
-	return hbbox;
-}
-
-/**
- * gnome_db_new_hpaned_widget
- */
-GtkWidget *
-gnome_db_new_hpaned_widget (void)
-{
-	GtkWidget *hpaned;
-
-	hpaned = gtk_hpaned_new ();
-	gtk_widget_show (hpaned);
-
-	return hpaned;
-}
-
-/**
- * gnome_db_new_hseparator_widget
- */
-GtkWidget *
-gnome_db_new_hseparator_widget (void)
-{
-	GtkWidget *hsep;
-
-	hsep = gtk_hseparator_new ();
-	gtk_widget_show (hsep);
-
-	return hsep;
-}
-
 /**
  * gnome_db_new_label_widget
+ * @text:
+ *
+ *
+ *
+ * Returns:
  */
 GtkWidget *
 gnome_db_new_label_widget (const gchar *text)
@@ -281,6 +206,10 @@ gnome_db_new_label_widget (const gchar *text)
 
 /**
  * gnome_db_new_notebook_widget
+ *
+ *
+ *
+ * Returns:
  */
 GtkWidget *
 gnome_db_new_notebook_widget (void)
@@ -298,6 +227,10 @@ gnome_db_new_notebook_widget (void)
 
 /**
  * gnome_db_new_option_menu_widget
+ *
+ *
+ *
+ * Returns:
  */
 GtkWidget *
 gnome_db_new_option_menu_widget (void)
@@ -310,41 +243,12 @@ gnome_db_new_option_menu_widget (void)
 	return option_menu;
 }
 
-#ifdef BUILD_WITH_GNOME
-/**
- * gnome_db_new_popup_menu
- */
-GtkWidget *
-gnome_db_new_popup_menu (GtkWidget *parent, GnomeUIInfo *ui_info, gpointer user_data)
-{
-	GtkWidget *popup_menu;
-
-        g_return_val_if_fail (parent != NULL, NULL);
-        g_return_val_if_fail (ui_info != NULL, NULL);
-
-        popup_menu = gnome_popup_menu_new (ui_info);
-        gnome_popup_menu_attach (popup_menu, parent, user_data);
-
-        return popup_menu;
-}
-#endif
-
-/**
- * gnome_db_new_radio_button_widget
- */
-GtkWidget *
-gnome_db_new_radio_button_widget (GSList *group, const gchar *label)
-{
-	GtkWidget *radio;
-
-	radio = gtk_radio_button_new_with_label (group, label);
-	gtk_widget_show (radio);
-
-	return radio;
-}
-
 /**
  * gnome_db_new_scrolled_window_widget
+ *
+ *
+ *
+ * Returns:
  */
 GtkWidget *
 gnome_db_new_scrolled_window_widget (void)
@@ -362,22 +266,15 @@ gnome_db_new_scrolled_window_widget (void)
 	return scroll;
 }
 
-/**
- * gnome_db_new_status_bar_widget
- */
-GtkWidget *
-gnome_db_new_status_bar_widget (void)
-{
-        GtkWidget* status;
-
-        status = gtk_statusbar_new ();
-        gtk_widget_show (status);
-
-        return status;
-}
-
 /**
  * gnome_db_new_table_widget
+ * @rows:
+ * @cols:
+ * @homogenous:
+ *
+ *
+ *
+ * Returns:
  */
 GtkWidget *
 gnome_db_new_table_widget (gint rows, gint cols, gboolean homogenous)
@@ -395,6 +292,11 @@ gnome_db_new_table_widget (gint rows, gint cols, gboolean homogenous)
 
 /**
  * gnome_db_new_text_widget
+ * @contents:
+ *
+ *
+ *
+ * Returns:
  */
 GtkWidget *
 gnome_db_new_text_widget (const gchar *contents)
@@ -416,24 +318,13 @@ gnome_db_new_text_widget (const gchar *contents)
 	return text;
 }
 
-/**
- * gnome_db_new_toolbar_widget
- */
-GtkWidget *
-gnome_db_new_toolbar_widget (GtkOrientation orientation, GtkToolbarStyle style)
-{
-	GtkWidget *toolbar;
-
-	toolbar = gtk_toolbar_new ();
-	gtk_toolbar_set_orientation (GTK_TOOLBAR (toolbar), orientation);
-	gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), style);
-	gtk_widget_show (toolbar);
-
-	return toolbar;
-}
-
 /**
  * gnome_db_new_tree_view_widget
+ * @model:
+ *
+ *
+ *
+ * Returns:
  */
 GtkWidget *
 gnome_db_new_tree_view_widget (GtkTreeModel *model)
@@ -451,6 +342,12 @@ gnome_db_new_tree_view_widget (GtkTreeModel *model)
 
 /**
  * gnome_db_new_vbox_widget
+ * @homogenous:
+ * @spacing:
+ *
+ *
+ *
+ * Returns:
  */
 GtkWidget *
 gnome_db_new_vbox_widget (gboolean homogenous, gint spacing)
@@ -463,64 +360,15 @@ gnome_db_new_vbox_widget (gboolean homogenous, gint spacing)
 	return vbox;
 }
 
-/**
- * gnome_db_new_vpaned_widget
- */
-GtkWidget *
-gnome_db_new_vpaned_widget (void)
-{
-	GtkWidget *vpaned;
-
-	vpaned = gtk_vpaned_new ();
-	gtk_widget_show (vpaned);
-
-	return vpaned;
-}
-
-/**
- * gnome_db_new_vseparator_widget
- */
-GtkWidget *
-gnome_db_new_vseparator_widget (void)
-{
-	GtkWidget *sep;
-
-	sep = gtk_vseparator_new ();
-	gtk_widget_show (sep);
-
-	return sep;
-}
-
-/**
- * gnome_db_set_widget_bg_color
- */
-void
-gnome_db_set_widget_bg_color (GtkWidget *widget, const gchar *color)
-{
-	GdkColor gdk_color;
-
-	g_return_if_fail (GTK_IS_WIDGET (widget));
-
-	gdk_color_parse (color, &gdk_color);
-	gtk_widget_modify_bg (widget, GTK_STATE_NORMAL, &gdk_color);
-}
-
-/**
- * gnome_db_set_widget_fg_color
- */
-void
-gnome_db_set_widget_fg_color (GtkWidget *widget, const gchar *color)
-{
-	GdkColor gdk_color;
-
-	g_return_if_fail (GTK_IS_WIDGET (widget));
-
-	gdk_color_parse (color, &gdk_color);
-	gtk_widget_modify_fg (widget, GTK_STATE_NORMAL, &gdk_color);
-}
 
 /**
  * gnome_db_option_menu_add_item
+ * @option_menu:
+ * @label:
+ *
+ *
+ *
+ * Returns:
  */
 GtkWidget *
 gnome_db_option_menu_add_item (GtkOptionMenu *option_menu, const gchar *label)
@@ -550,6 +398,12 @@ gnome_db_option_menu_add_item (GtkOptionMenu *option_menu, const gchar *label)
 
 /**
  * gnome_db_option_menu_add_stock_item
+ * @option_menu:
+ * @stock_id:
+ *
+ *
+ *
+ * Returns:
  */
 GtkWidget *
 gnome_db_option_menu_add_stock_item (GtkOptionMenu *option_menu, const gchar *stock_id)
@@ -579,6 +433,8 @@ gnome_db_option_menu_add_stock_item (GtkOptionMenu *option_menu, const gchar *st
 
 /**
  * gnome_db_option_menu_add_separator
+ * @option_menu:
+ *
  */
 void
 gnome_db_option_menu_add_separator (GtkOptionMenu *option_menu)
@@ -603,6 +459,11 @@ gnome_db_option_menu_add_separator (GtkOptionMenu *option_menu)
 
 /**
  * gnome_db_option_menu_get_selection
+ * @option_menu:
+ *
+ *
+ *
+ * Returns:
  */
 const gchar *
 gnome_db_option_menu_get_selection (GtkOptionMenu *option_menu)
@@ -625,6 +486,9 @@ gnome_db_option_menu_get_selection (GtkOptionMenu *option_menu)
 
 /**
  * gnome_db_option_menu_set_selection
+ * @option_menu:
+ * @selection:
+ *
  */
 void
 gnome_db_option_menu_set_selection (GtkOptionMenu *option_menu, const gchar *selection)
@@ -680,6 +544,7 @@ gnome_db_text_clear (GtkTextView *text)
 
 /**
  * gnome_db_text_copy_clipboard
+ * @text:
  */
 void
 gnome_db_text_copy_clipboard (GtkTextView *text)
@@ -691,6 +556,8 @@ gnome_db_text_copy_clipboard (GtkTextView *text)
 
 /**
  * gnome_db_text_cut_clipboard
+ * @text:
+ *
  */
 void
 gnome_db_text_cut_clipboard (GtkTextView *text)
@@ -703,6 +570,11 @@ gnome_db_text_cut_clipboard (GtkTextView *text)
 
 /**
  * gnome_db_text_get_char_count
+ * @text:
+ *
+ *
+ *
+ * Returns:
  */
 gint
 gnome_db_text_get_char_count (GtkTextView *text)
@@ -713,6 +585,11 @@ gnome_db_text_get_char_count (GtkTextView *text)
 
 /**
  * gnome_db_text_get_line_count
+ * @text:
+ *
+ *
+ *
+ * Returns:
  */
 gint
 gnome_db_text_get_line_count (GtkTextView *text)
@@ -723,6 +600,11 @@ gnome_db_text_get_line_count (GtkTextView *text)
 
 /**
  * gnome_db_text_get_text
+ * @text:
+ *
+ *
+ *
+ * Returns:
  */
 gchar *
 gnome_db_text_get_text (GtkTextView *text)
@@ -746,6 +628,9 @@ gnome_db_text_get_text (GtkTextView *text)
 
 /**
  * gnome_db_text_insert_at_cursor
+ * @text:
+ * @contents:
+ * @len:
  */
 void
 gnome_db_text_insert_at_cursor (GtkTextView *text, const gchar *contents, gint len)
@@ -758,6 +643,7 @@ gnome_db_text_insert_at_cursor (GtkTextView *text, const gchar *contents, gint l
 
 /**
  * gnome_db_text_paste_clipboard
+ * @text:
  */
 void
 gnome_db_text_paste_clipboard (GtkTextView *text)
@@ -771,6 +657,10 @@ gnome_db_text_paste_clipboard (GtkTextView *text)
 
 /**
  * gnome_db_text_set_text
+ * @text:
+ * @contents:
+ * @len:
+ *
  */
 void
 gnome_db_text_set_text (GtkTextView *text, const gchar *contents, gint len)
@@ -781,6 +671,12 @@ gnome_db_text_set_text (GtkTextView *text, const gchar *contents, gint len)
 
 /**
  * gnome_db_select_file_dialog
+ * @parent:
+ * @title:
+ *
+ *
+ *
+ * Returns:
  */
 gchar *
 gnome_db_select_file_dialog (GtkWidget *parent, const gchar *title)
@@ -788,6 +684,13 @@ gnome_db_select_file_dialog (GtkWidget *parent, const gchar *title)
 	GtkWidget *filesel;
         gchar *res = NULL;
 
+#if GTK_CHECK_VERSION(2,4,0)
+	/* create dialog */
+	filesel = gtk_file_chooser_dialog_new (title, parent, GTK_FILE_CHOOSER_ACTION_SAVE,
+					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					       GTK_STOCK_OK, GTK_RESPONSE_OK,
+					       NULL);
+#else
         /* create dialog */
         filesel = gtk_file_selection_new (title);
 	if (GTK_IS_WINDOW (parent)) {
@@ -795,10 +698,17 @@ gnome_db_select_file_dialog (GtkWidget *parent, const gchar *title)
 				              GTK_WINDOW (parent));
 	}
         gtk_file_selection_show_fileop_buttons (GTK_FILE_SELECTION (filesel));
+#endif
+
+	gtk_dialog_set_default_response (GTK_DIALOG (filesel), GTK_RESPONSE_OK);
 
         /* wait for selection from user */
 	if (gtk_dialog_run (GTK_DIALOG (filesel)) == GTK_RESPONSE_OK)
+#if GTK_CHECK_VERSION(2, 4, 0)
+		res = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (filesel));
+#else
 		res = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (filesel)));
+#endif
         else
 		res = NULL;
 
@@ -810,6 +720,14 @@ gnome_db_select_file_dialog (GtkWidget *parent, const gchar *title)
 
 /**
  * gnome_db_new_alert
+ * @parent:
+ * @type:
+ * @primary_text:
+ * @secondary_text:
+ *
+ *
+ *
+ * Returns:
  */
 GtkWidget *
 gnome_db_new_alert (GtkWindow *parent,
@@ -890,6 +808,9 @@ gnome_db_new_alert (GtkWindow *parent,
 
 /**
  * gnome_db_show_error
+ * @format:
+ * @...:
+ *
  */
 void
 gnome_db_show_error (const gchar *format, ...)
@@ -916,6 +837,7 @@ gnome_db_show_error (const gchar *format, ...)
 /**
  * gnome_db_show_message
  * @format: Format string.
+ * @...:
  *
  * Displays an information message to the user.
  */
