@@ -19,6 +19,7 @@
  * USA
  */
 
+#include <gtk/gtk.h>
 #include "marshal.h"
 #include <libgnomedb/gnome-db-database.h>
 #include <libgnomedb/gnome-db-table.h>
@@ -467,10 +468,10 @@ gnome_db_canvas_db_relations_new (GnomeDbDict *dict, GnomeDbGraph *graph)
 	GnomeDbDatabase *db;
 	GnomeDbCanvasDbRelations *canvas;
 
-	g_return_val_if_fail (dict && IS_GNOME_DB_DICT (dict), NULL);
+	g_return_val_if_fail (!dict || IS_GNOME_DB_DICT (dict), NULL);
 	if (graph) {
 		g_return_val_if_fail (IS_GNOME_DB_GRAPH (graph), NULL);
-		g_return_val_if_fail (dict == gnome_db_base_get_dict (GNOME_DB_BASE (graph)), NULL);
+		g_return_val_if_fail (ASSERT_DICT (dict) == gnome_db_base_get_dict (GNOME_DB_BASE (graph)), NULL);
 	}
 
         obj = g_object_new (GNOME_DB_CANVAS_DB_RELATIONS_TYPE, "aa", FALSE, NULL);
@@ -478,7 +479,7 @@ gnome_db_canvas_db_relations_new (GnomeDbDict *dict, GnomeDbGraph *graph)
 	canvas = GNOME_DB_CANVAS_DB_RELATIONS (obj);
 
 	/* connecting to the GnomeDbDatabase */
-	db = gnome_db_dict_get_database (dict);
+	db = gnome_db_dict_get_database (ASSERT_DICT (dict));
 	canvas->priv->db = db;
 	gnome_db_base_connect_nullify (db, G_CALLBACK (db_nullified_cb), obj);
 	g_signal_connect (G_OBJECT (db), "constraint_added",
