@@ -32,6 +32,7 @@
 #include <libgnomedb/gnome-db-server-data-type.h>
 #include <libgnomedb/gnome-db-parameter.h>
 #include <libgnomedb/gnome-db-data-set.h>
+#include <libgnomedb/utility.h>
 
 #define GNOME_DB_DATA_CELL_RENDERER_COMBO_PATH "gnome-db-data-cell-renderer-combo-path"
 
@@ -87,7 +88,7 @@ enum {
 
 struct _GnomeDbDataCellRendererComboPrivate
 {
-	GnomeDbEntryCombo  *combo; /* only the attributes of that widget are used! */
+	DataSetNodeInfo    *info;
 	GnomeDbDataModel   *data_model;
 	guint               focus_out_id;
 
@@ -200,9 +201,9 @@ gnome_db_data_cell_renderer_combo_dispose (GObject *object)
 {
 	GnomeDbDataCellRendererCombo *datacell = GNOME_DB_DATA_CELL_RENDERER_COMBO (object);
 
-	if (datacell->priv->combo) {
-		gtk_widget_destroy (GTK_WIDGET (datacell->priv->combo));
-		datacell->priv->combo = NULL;
+	if (datacell->priv->info) {
+		utility_data_set_node_info_destroy (datacell->priv->info);
+		datacell->priv->info = NULL;
 	}
 
 	if (datacell->priv->data_model) {
@@ -281,24 +282,27 @@ gnome_db_data_cell_renderer_combo_set_property (GObject      *object,
 					gvalues = g_list_next (gvalues);
 				}
 
-				g_return_if_fail (length == datacell->priv->combo->ref_n_cols);
+				g_return_if_fail (length == datacell->priv->info->ref_n_cols);
 				
 				if (allnull) 
 					g_object_set (G_OBJECT (object), "text", "", NULL);
 				else {
 					/* find the data model row for the values */
+					if (gnome_db_data_model_get_status (datacell->priv->data_model) & 
+					    GNOME_DB_DATA_MODEL_NEEDS_INIT_REFRESH)
+						gnome_db_data_model_refresh (datacell->priv->data_model, NULL);
 					row = gnome_db_data_model_get_row_from_values (datacell->priv->data_model, values,
-										       datacell->priv->combo->ref_cols_index);
+										       datacell->priv->info->ref_cols_index);
 					if (row >= 0) {
 						GList *dsplay_values = NULL;
 						gint i;
 						gchar *str;
 						
-						for (i = 0; i < datacell->priv->combo->shown_n_cols; i++) {
+						for (i = 0; i < datacell->priv->info->shown_n_cols; i++) {
 							const GdaValue *value;
 							
 							value = gda_data_model_get_value_at (GDA_DATA_MODEL (datacell->priv->data_model),
-											     datacell->priv->combo->shown_cols_index [i],
+											     datacell->priv->info->shown_cols_index [i],
 											     row);
 							dsplay_values = g_list_append (dsplay_values, value);
 						}
@@ -307,8 +311,12 @@ gnome_db_data_cell_renderer_combo_set_property (GObject      *object,
 						g_object_set (G_OBJECT (object), "text", str, NULL);
 						g_free (str);
 					}
-					else
-						g_object_set (G_OBJECT (object), "text", "???", NULL);
+					else {
+						if (datacell->priv->attributes & GNOME_DB_VALUE_CAN_BE_NULL)
+							g_object_set (G_OBJECT (object), "text", "", NULL);
+						else
+							g_object_set (G_OBJECT (object), "text", "???", NULL);
+					}
 				}
 
 				g_slist_free (values);
@@ -330,8 +338,8 @@ gnome_db_data_cell_renderer_combo_set_property (GObject      *object,
 			gchar *str;
 
 			/* extract the values which need to be displayed */
-			for (i = 0; i < datacell->priv->combo->shown_n_cols; i++) {
-				value = g_list_nth_data (gvalues, datacell->priv->combo->shown_cols_index [i]);
+			for (i = 0; i < datacell->priv->info->shown_n_cols; i++) {
+				value = g_list_nth_data (gvalues, datacell->priv->info->shown_cols_index [i]);
 				dsplay_values = g_list_append (dsplay_values, value);
 			}
 			
@@ -420,7 +428,7 @@ gnome_db_data_cell_renderer_combo_new (GnomeDbDataSet *data_set, GnomeDbDataSetN
 
 	datacell->priv->data_model = node->data_for_param;
 	g_object_ref (datacell->priv->data_model);
-	datacell->priv->combo = GNOME_DB_ENTRY_COMBO (gnome_db_entry_combo_new (data_set, node));
+	datacell->priv->info = utility_data_set_node_info_create (data_set, node);
 	
 	return GTK_CELL_RENDERER (obj);
 }
@@ -530,14 +538,15 @@ gnome_db_data_cell_renderer_combo_start_editing (GtkCellRenderer     *cell,
 
 	combo = gnome_db_combo_new_with_model (gnome_db_base_get_dict (GNOME_DB_BASE (datacell->priv->data_model)),
 					       GDA_DATA_MODEL (datacell->priv->data_model),
-					       datacell->priv->combo->shown_n_cols, 
-					       datacell->priv->combo->shown_cols_index);
+					       datacell->priv->info->shown_n_cols, 
+					       datacell->priv->info->shown_cols_index);
 	
 	g_object_set (combo, "has_frame", FALSE, NULL);
 	g_object_set_data_full (G_OBJECT (combo),
 				GNOME_DB_DATA_CELL_RENDERER_COMBO_PATH,
 				g_strdup (path), g_free);
-	gnome_db_combo_add_undef_choice (GNOME_DB_COMBO (combo), datacell->priv->attributes & GNOME_DB_VALUE_CAN_BE_NULL);
+	gnome_db_combo_add_undef_choice (GNOME_DB_COMBO (combo),
+					 (datacell->priv->attributes & GNOME_DB_VALUE_CAN_BE_NULL) ? TRUE : FALSE);
 	gtk_widget_show (combo);
 
 	g_signal_connect (GTK_CELL_EDITABLE (combo), "editing_done",
@@ -569,8 +578,8 @@ gnome_db_data_cell_renderer_combo_editing_done (GtkCellEditable *combo, GnomeDbD
 		return;
 	
 	list = gnome_db_combo_get_values_ext (GNOME_DB_COMBO (combo), 
-					      datacell->priv->combo->ref_n_cols, 
-					      datacell->priv->combo->ref_cols_index);
+					      datacell->priv->info->ref_n_cols, 
+					      datacell->priv->info->ref_cols_index);
 	list_all = gnome_db_combo_get_values_ext (GNOME_DB_COMBO (combo), 0, NULL);
 
 	path = g_object_get_data (G_OBJECT (combo), GNOME_DB_DATA_CELL_RENDERER_COMBO_PATH);
