@@ -31,6 +31,7 @@
 #include <libgnomedb/gnome-db-renderer.h>
 #include <libgnomedb/gnome-db-result-set.h>
 #include <libgnomedb/libgnomedb.h>
+#include <libgnomedb/utility.h>
 
 static void gnome_db_entry_combo_class_init (GnomeDbEntryComboClass *class);
 static void gnome_db_entry_combo_init (GnomeDbEntryCombo *wid);
@@ -94,6 +95,7 @@ typedef struct {
 struct  _GnomeDbEntryComboPriv {
         GtkWidget          *combo_entry;
 	GSList             *combo_nodes; /* list of ComboNode structures */
+	DataSetNodeInfo    *info;
 	
 	gboolean            data_valid;
 	gboolean            null_forced;
@@ -215,9 +217,6 @@ gnome_db_entry_combo_init (GnomeDbEntryCombo *combo)
 	combo->priv->show_actions = TRUE;
 }
 
-static void compute_shown_columns_index (GnomeDbEntryCombo *combo);
-static void compute_ref_columns_index (GnomeDbEntryCombo *combo);
-
 /**
  * gnome_db_entry_combo_new
  * @data_set: a #GnomeDbDataSet object
@@ -250,7 +249,6 @@ gnome_db_entry_combo_new (GnomeDbDataSet *data_set, GnomeDbDataSetNode *node)
 	obj = g_object_new (GNOME_DB_ENTRY_COMBO_TYPE, NULL);
 	combo = GNOME_DB_ENTRY_COMBO (obj);
 
-	combo->data_model = node->data_for_param;
 	g_object_ref (node->data_for_param);
 	if (gnome_db_data_model_get_status (node->data_for_param) & 
 	    (GNOME_DB_DATA_MODEL_CAN_BE_REFRESHED | GNOME_DB_DATA_MODEL_NEEDS_INIT_REFRESH))
@@ -278,14 +276,12 @@ gnome_db_entry_combo_new (GnomeDbDataSet *data_set, GnomeDbDataSetNode *node)
 	}
 	combo->priv->null_possible = null_possible;
 
-	/* determine which column will be shown, and which will serve as reference */
-	compute_shown_columns_index (combo);
-	compute_ref_columns_index (combo);
+	combo->priv->info = utility_data_set_node_info_create (data_set, node);
 
 	/* create the combo box itself */
 	entry = gnome_db_combo_new_with_model (gnome_db_base_get_dict (GNOME_DB_BASE (data_set)), 
 					       GDA_DATA_MODEL (node->data_for_param), 
-					       combo->shown_n_cols, combo->shown_cols_index);
+					       combo->priv->info->shown_n_cols, combo->priv->info->shown_cols_index);
 	g_signal_connect (G_OBJECT (entry), "changed",
 			  G_CALLBACK (combo_contents_changed_cb), combo);
 
@@ -300,102 +296,6 @@ gnome_db_entry_combo_new (GnomeDbDataSet *data_set, GnomeDbDataSetNode *node)
 	return GTK_WIDGET (obj);
 }
 
-static void
-compute_shown_columns_index (GnomeDbEntryCombo *combo)
-{
-	gint ncols, nparams;
-	gint *mask = NULL, masksize = 0;
-
-	nparams = g_slist_length (combo->priv->combo_nodes);
-	g_return_if_fail (nparams > 0);
-	ncols = gda_data_model_get_n_columns (GDA_DATA_MODEL (combo->data_model));
-	g_return_if_fail (ncols > 0);	
-
-	if (ncols > nparams) {
-		/* we only want columns which are not parameter values */
-		gint i, current = 0;
-		
-		masksize = ncols - nparams;
-		mask = g_new0 (gint, masksize);
-		for (i=0; i<ncols ; i++) {
-			GSList *list = combo->priv->combo_nodes;
-			gboolean found = FALSE;
-			while (list && !found) {
-				if (COMBO_NODE (list->data)->position == i)
-					found = TRUE;
-				else
-					list = g_slist_next (list);
-			}
-			if (!found) {
-				mask[current] = i;
-				current ++;
-			}
-		}
-		masksize = current;
-	}
-	else {
-		/* we want all the columns */
-		gint i;
-
-		masksize = ncols;
-		mask = g_new0 (gint, masksize);
-		for (i=0; i<ncols; i++) {
-			mask[i] = i;
-		}
-	}
-
-	combo->shown_n_cols = masksize;
-	combo->shown_cols_index = mask;
-}
-
-static void
-compute_ref_columns_index (GnomeDbEntryCombo *combo)
-{
-	gint ncols, nparams;
-	gint *mask = NULL, masksize = 0;
-
-	nparams = g_slist_length (combo->priv->combo_nodes);
-	g_return_if_fail (nparams > 0);
-	ncols = gda_data_model_get_n_columns (GDA_DATA_MODEL (combo->data_model));
-	g_return_if_fail (ncols > 0);	
-
-	if (ncols > nparams) {
-		/* we only want columns which are parameters values */
-		gint i, current = 0;
-		
-		masksize = ncols - nparams;
-		mask = g_new0 (gint, masksize);
-		for (i=0; i<ncols ; i++) {
-			GSList *list = combo->priv->combo_nodes;
-			gboolean found = FALSE;
-			while (list && !found) {
-				if (COMBO_NODE (list->data)->position == i)
-					found = TRUE;
-				else
-					list = g_slist_next (list);
-			}
-			if (found) {
-				mask[current] = i;
-				current ++;
-			}
-		}
-		masksize = current;
-	}
-	else {
-		/* we want all the columns */
-		gint i;
-
-		masksize = ncols;
-		mask = g_new0 (gint, masksize);
-		for (i=0; i<ncols; i++) {
-			mask[i] = i;
-		}
-	}
-
-	combo->ref_n_cols = masksize;
-	combo->ref_cols_index = mask;
-}
-
 static void
 choose_auto_default_value (GnomeDbEntryCombo *combo)
 {
@@ -417,23 +317,11 @@ gnome_db_entry_combo_dispose (GObject *object)
 	combo = GNOME_DB_ENTRY_COMBO (object);
 
 	if (combo->priv) {		
-		if (combo->data_model) {
-			g_object_unref (combo->data_model);
-			combo->data_model = NULL;
+		if (combo->priv->info) {
+			utility_data_set_node_info_destroy (combo->priv->info);
+			combo->priv->info = NULL;
 		}
-		
-		if (combo->shown_cols_index) {
-			combo->shown_n_cols = 0;
-			g_free (combo->shown_cols_index);
-			combo->shown_cols_index = NULL;
-		}
-		
-		if (combo->ref_cols_index) {
-			combo->ref_n_cols = 0;
-			g_free (combo->ref_cols_index);
-			combo->ref_cols_index = NULL;
-		}
-		
+				
 		if (combo->priv->combo_nodes) {
 			GSList *list= combo->priv->combo_nodes;
 			while (list) {
@@ -503,8 +391,10 @@ gnome_db_entry_combo_get_property (GObject              *object,
 static void
 combo_contents_changed_cb (GnomeDbCombo *entry, GnomeDbEntryCombo *combo)
 {
-	if (gnome_db_combo_undef_selected (GNOME_DB_COMBO (combo->priv->combo_entry))) /* Set to NULL? */
+	if (gnome_db_combo_undef_selected (GNOME_DB_COMBO (combo->priv->combo_entry))) /* Set to NULL? */ {
 		gnome_db_entry_combo_set_values (combo, NULL);
+		gnome_db_entry_combo_emit_signal (combo);
+	}
 	else {
 		GtkTreeIter iter;
 		GSList *list;
@@ -522,9 +412,9 @@ combo_contents_changed_cb (GnomeDbCombo *entry, GnomeDbEntryCombo *combo)
 		while (list) {
 			ComboNode *node = COMBO_NODE (list->data);
 
-			gtk_tree_model_get (model, &iter, node->position, & (node->value), -1);
-			g_print ("%s(): Set Combo Node value to %s\n", __FUNCTION__,
-				 node->value ? gda_value_stringify (node->value) : "Null");
+			gtk_tree_model_get (model, &iter, node->position, &(node->value), -1);
+			/*g_print ("%s(): Set Combo Node value to %s\n", __FUNCTION__,
+			  node->value ? gda_value_stringify (node->value) : "Null");*/
 			list = g_slist_next (list);
 		}
 		
@@ -579,7 +469,7 @@ gnome_db_entry_combo_set_values (GnomeDbEntryCombo *combo, GSList *values)
 		
 		model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo->priv->combo_entry));
 		if (gnome_db_data_proxy_get_iter_from_values (GNOME_DB_DATA_PROXY (model), &iter, 
-							      values, combo->ref_cols_index)) {
+							      values, combo->priv->info->ref_cols_index)) {
 			ComboNode *node;
 
 			real_combo_block_signals (combo);
@@ -710,16 +600,16 @@ gnome_db_entry_combo_set_values_orig (GnomeDbEntryCombo *combo, GSList *values)
 		nodes = combo->priv->combo_nodes;
 		argptr = values;
 		while (argptr && nodes && equal) {
-			GdaDataModelColumnAttributes *attrs;
+			GdaColumn *attrs;
 			GdaValueType type=GDA_VALUE_TYPE_NULL;
 			
-			attrs = gda_data_model_describe_column (combo->data_model, 
+			attrs = gda_data_model_describe_column (combo->priv->info->data_model, 
 								COMBO_NODE (nodes->data)->position);
 			arg_value = (GdaValue*) (argptr->data);
 			
 			if (arg_value)
 				type = gda_value_get_type (arg_value);
-			equal = (type == attrs->gda_type);
+			equal = (type == gda_column_get_gdatype (attrs));
 			
 			nodes = g_slist_next (nodes);
 			argptr = g_slist_next (argptr);
