@@ -676,7 +676,7 @@ build_page (MainConfig *dict)
 
 	label = gtk_label_new (NULL);
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
-	str = g_strdup_printf ("<b>%s</b>", _("Available dictionnary objects:"));
+	str = g_strdup_printf ("<b>%s</b>", _("Available dictionary objects:"));
 	gtk_label_set_markup (GTK_LABEL (label), str);
 	g_free (str);
 	gtk_box_pack_start (GTK_BOX (vb), label, FALSE, TRUE, 0);
